;----------------------------------------------------------------
; NAME: harm_tseries
;
; PURPOSE: Finds 1st, 1st+2nd, 1st+2nd+3rd harmonic time series.
;
; LANGUAGE: IDL
;
; INPUTS:
;          name     type      dimension      description
;     -------------------------------------------------------
;          ts       float     1 x t          Time series
;          dt       float     scalar         time increment in time series ts
;          period1  float     scalar         period of 1st harmonic.
;
; OUTPUT:
;
;          tsh      float     t x 3          time series for 1st, 1st+2nd, 1st+2nd+3rd harmonic
;          phase    float     1 x 3          phase for first three harmonics
;          ampl     float     1 x 3          amplitude for first three harmonics
;          pvar     float     1 x 3          percentage variance for first three harmonics
;
;          t => Length of time series
;
; EXTERNALS:
;             lsfcos        -> Finds harmonic coefficients using least squares fitting.
;             harmonic_wave -> Constructs time series using harmonic wave formulas.
;
; NOTES:  dt and period1 must have same time unit.  For example, to
;         describe the seasonal cycle for daily data, dt = 1. and
;         period1 = 365.25.
;
; CREATION HISTORY:
;                    Darren Jackson CIRES/ETL June 2002
;-----------------------------------------------------------------------
PRO harm_tseries, ts, dt, period1, tsh, phase, ampl ,pvar
;
; Set parameters
;
t1   = 0.0    
ifit = 1
a=FLTARR(3)
b=FLTARR(3)
c=FLTARR(3)
p=FLTARR(3)
w=FLTARR(3)
phase=FLTARR(3)
ampl=FLTARR(3)
pvar=FLTARR(3)
;
; Identify length of input time series
;
result_ts = SIZE(ts)
ts_length = result_ts(1)
tsh = FLTARR(ts_length,3)
;
; Compute frequency and period for second and third harmonics
;
p(0) = period1
w(0) = 2. * !pi / p(0)
FOR n = 1, 2 DO BEGIN
   w(n) = 2. * w(0)
   p(n) = p(0) / (n+1)
ENDFOR
;
; Compute harmonic coefficients a,b,c for first three harmonics
;
LSFCOS, ifit, ts, dt, t1, p(0), a1, b1, c1
LSFCOS, ifit, ts, dt, t1, p(1), a2, b2, c2
LSFCOS, ifit, ts, dt, t1, p(2), a3, b3, c3
a(0)=a1
a(1)=a2
a(2)=a3
b(0)=b1
b(1)=b2
b(2)=b3
c(0)=c1
c(1)=c2
c(2)=c3
;
; Compute phase and amplitude for each harmonic
;
FOR n = 0,2 DO BEGIN
  IF(a(n) gt 0.) THEN BEGIN
     phase(n) = ATAN(b(n)/a(n)) * !radeg
     IF(phase(n) lt 0.) THEN phase(n) = phase(n) + 360.
  ENDIF
  IF(a(n) lt 0.) THEN phase(n) = ATAN(b(n)/a(n)) * !radeg + 180.
  IF(a(n) eq 0.) THEN phase(n) = 90.
  ampl(n) = SQRT(a(n)^2.+b(n)^2.)
ENDFOR
;
; Reconstruct 1st, 1st+2nd, and 1st+2nd+3rd harmonic time series
;
tsh(*,0) = HARMONIC_WAVE(a(0),b(0),c(0),w(0),ts_length,dt)
tsh(*,1) = HARMONIC_WAVE(a(1),b(1),0.0,w(1),ts_length,dt)
tsh(*,1) = tsh(*,1) + tsh(*,0)
tsh(*,2) = HARMONIC_WAVE(a(2),b(2),0.0,w(2),ts_length,dt)
tsh(*,2) = tsh(*,2) + tsh(*,1)
;
; Compute percentage variance of each harmonic
;
tmp = 0.
FOR n=0,2 DO BEGIN
   tmp = tmp + (a(n)^2. + b(n)^2.)/2.
ENDFOR
pvar(0)=(a(0)^2. + b(0)^2.)/2. / tmp * 100.
pvar(1)=(a(1)^2. + b(1)^2.)/2. / tmp * 100.
pvar(2)=(a(2)^2. + b(2)^2.)/2. / tmp * 100.

END
