;------------------------------------------------------------
; NAME: lsfcos
;
; PURPOSE: Compute harmonic coefficients for time series
;
; LANGUAGE: IDL
;
; INPUTS:
;         name     type     dimension     description
;      -----------------------------------------------------
;        ifit      short    scalar        0 = fits harmonic function with constant
;                                         1 = fits harmonic function w/o constant
;        ts        float    1 x t         time series
;        dt        float    scalar        increment of time series
;        t1        float    scalar        time of first measurement
;        period    float    scalar        period
;
; OUTPUTS:
;
;        a         float    scalar        harmonic coefficient for cosine
;        b         float    scalar        harmonic coefficient for sine
;        c         float    scalar        harmonic coefficient for constant
;
; NOTES:
;        Code based on fortran 77 code written by
;          jon ahlquist, dept. of meteorology, 262-0794.
;          19 feb 1981, 5 mar 1981
;        jon's code based on
;          exercise 2.3 in bloomfield, fourier analysis of time series: an introduction,  
;          copyright 1976, page 16.                                       
;                      
; CREATION HISTORY:  Darren Jackson  CIRES/ETL June 2002
;-----------------------------------------------------------------------
PRO lsfcos, ifit, ts, dt, t1, period, a, b, c
;
; Find length of time series
;
result = SIZE(ts)
n = result(1)

w = 2.* !pi * dt / period
sum2c = SUMCOS(n,2.*w)
sum2s = SUMSIN(n,2.*w)
sum1c = SUMCOS(n,w)
sum1s = SUMSIN(n,w)
;
; Fill 3x3 matrix
;
p=FLTARR(3,3)
amatrix=FLTARR(3,3)
bmatrix=FLTARR(3,3)
cmatrix=FLTARR(3,3)

p(0,0) = 0.5 * (n + sum2c)
p(0,1) = 0.5 * sum2s
p(0,2) = sum1c
p(1,0) = p(0,1)
p(1,1) = 0.5 * (n - sum2c)
p(1,2) = sum1s
p(2,0) = sum1s
p(2,1) = sum1c
p(2,2) = n

q1 = 0.
q2 = 0.
q3 = 0.
FOR i=0,n-1 DO BEGIN
   q1 = q1 + ts(i)*cos(w*i)
   q2 = q2 + ts(i)*sin(w*i)
   q3 = q3 + ts(i)
ENDFOR

amatrix(0,0)=q1
amatrix(0,1)=p(0,1)
amatrix(0,2)=p(0,2)
amatrix(1,0)=q2
amatrix(1,1)=p(1,1)
amatrix(1,2)=p(1,2)
amatrix(2,0)=q3
amatrix(2,1)=p(2,1)
amatrix(2,2)=p(2,2)

bmatrix(0,0)=p(0,0)
bmatrix(0,1)=q1
bmatrix(0,2)=p(0,2)
bmatrix(1,0)=p(1,0)
bmatrix(1,1)=q2
bmatrix(1,2)=p(1,2)
bmatrix(2,0)=p(2,0)
bmatrix(2,1)=q3
bmatrix(2,2)=p(2,2)

cmatrix(0,0)=p(0,0)
cmatrix(0,1)=p(0,1)
cmatrix(0,2)=q1
cmatrix(1,0)=p(1,0)
cmatrix(1,1)=p(1,1)
cmatrix(1,2)=q2
cmatrix(2,0)=p(2,0)
cmatrix(2,1)=p(2,1)
cmatrix(2,2)=q3

CASE ifit OF
   0 : BEGIN
       a = (q1*p(1,1) - q2*p(0,1))/(p(0,0)*p(1,1) - p(1,2)*p(0,1))
       b = (p(0,0)*q2 - p(1,0)*q1)/(p(0,0)*p(1,1) - p(2,1)*p(0,1))
   END
   1 : BEGIN
       det = DETERM(p)
       a   = DETERM(amatrix)/det
       b   = DETERM(bmatrix)/det
       c   = DETERM(cmatrix)/det
   END
   ELSE:  PRINT, "ifit is illegal value"
ENDCASE
;
; Shift a and b for the case when t1 in nonzero
;
temp = a
a = a*COS(w*t1/dt) - b    * SIN(w*t1/dt)
b = b*COS(w*t1/dt) + temp * SIN(w*t1/dt)

END

 


