;------------------------------------------------------------------------------
;+
; NAME:
;       OVERPLOTARR
; PURPOSE:
;       Plot arrows with specified u,v (delta-x,delta-y) components on top of 
;       an image plotted with image.pro.  Requires that a regular grid defines 
;       the arrow origin points.  
; CATEGORY:
; LAST UPDATE:
;       19 April 2000 
; CALLING SEQUENCE:
;       overplotarr    -> Displays help information for overplot.pro.
;       overplotarr, uarray, varray
; INPUTS:
;       uarray  Two-dimensional array of u (delta x) vector components
;       varray  Two-dimensional array of v (delta y) vector components 
; OUTPUTS:
; MODIFICATION HISTORY:
;       D. Anderson, March 1999 
;       D. Anderson, April 2000  Added PNG option 
;       D. Anderson, April 2000  Replace velovect call with velovect2 call 
;       D. Anderson, Nov 2000    Added /multi keyword option 
;
; NOTES:
;   (1) If using in conjunction with pimage or pcontour to create Postscript 
;       or PNG output, make sure that each call to pimage or pcontour preceding 
;       a call to this procedure includes the /multi keyword. 
;
; VARIABLE DESCRIPTIONS:
;   px   Lower-left x coordinate for plotted image, in full page coordiates
;   py   Lower-left y coordinate for plotted image, in full page coordiates
;   pxc  Lower-left x coordinate for plotted colorbar, in full page coordiates
;   pyc  Lower-left y coordinate for plotted colorbar, in full page coordiates
;   xs   Width of plotted image, in full page coordiates
;   ys   Height of plotted image, in full page coordiates
;   xsc  Width of plotted colorbar, in full page coordiates
;   ysc  Height of plotted colorbar, in full page coordiates
;-----------------------------------------------------------------------------
pro overplotarr, uarray, varray, project=prj, $
                 minlat=minlat, maxlat=maxlat, minlon=minlon, maxlon=maxlon, $
                 order=order, sublat=sublat, sublng=sublng, coast=coast, $
                 usa=usa, _extra=xtra, pngout=pngout, pngfile=gfile , $
                 cmin=cmn, cmax=cmx, vmin=vmn, vmax=vmx, region=region, $
                 multi=multi

if (n_params(0) EQ 0) then begin
  print,' OVERPLOTARR   -> Displays help information for image.pro'
  print,' OVERPLOTARR, u, v -> Plots data on top of image plotted with pimage.pro or pcontour.pro.'
  print,' Keywords:'
  print,'   PROJECT=prj      Map projection (default: cylindrical equidistant)' 
  print,'   MINLAT=minlat    Minimum latitude value.'
  print,'   MAXLAT=maxlat    Maximum latitude value.'
  print,'   MINLON=minlon    Minimum longitude value.'
  print,'   MAXLON=maxlon    Maximum longitude value.'
  print,'   REGION=region    [minlat,maxlat,minlon,maxlon] array.'
  print,'   VMIN=vmn         Minimum data value.'
  print,'   VMAX=vmx         Maximum data value.'
  print,'   CMIN=cmn         Color value corresponding to minimum data value.'
  print,'   CMAX=cmx         Color value corresponding to maximum data value.'
  print,'   SUBLAT=sublat    Sub-satellite latitude for satellite projection.'
  print,'   SUBLNG=sublng    Sub-satellite longitude for satellite projection.'
  print,'   COLOR=color      Color index value for arrows.'
  print,'   MISSING=missing  uarray or varray values equal to missing are ignored.'
  print,'   DOTS=dots        Set to 1 to place dot at each missing point, 0 to'
  print,'                    place nothing at missing points.  (Missing must be'
  print,'                    specified for this to take effect).' 
  print,'   LENGTH=length    Length factor.  Default 1.0 makes longest vector' 
  print,'                    the length of a cell.' 
  print,'   PNGFILE=gfile    Name of output PNG file.' 
  print,'   (ETC.)           All other keywords accepted by VELOVECT.' 
  print,'   /COAST           Draw coastlines.' 
  print,'   /PNGOUT          Produce PNG output.' 
  print,'   /USA             Draw USA state boundaries.' 
  print,'   /ORDER           Reverse order of rows.'
  return
endif

 if n_elements(prj) eq 0 then prj='cyl' 
 if n_elements(region) eq 4 then begin
   minlat = region(0)
   maxlat = region(1)
   minlon = region(2)
   maxlon = region(3)
   if (minlat gt maxlat) OR (minlon gt maxlon) then begin
     print, 'Error: problem with specified values for region!'
     stop
   endif
 endif
 if n_elements(ocol) eq 0 then ocol=1  
 if n_elements(dots) eq 0 then dots=0
 if n_elements(psym) eq 0 then psym=1  
 if n_elements(symsize) eq 0 then symsize=1  
 if n_elements(sublat) eq 0 then sublat=0.0
 if n_elements(sublng) eq 0 then sublng=-135.0
 if n_elements(gfile) eq 0 then gfile='idl.png'
 if n_elements(cmn) eq 0 then begin
   defsysv, '!CMIN', exists=i
   if i eq 1 then cmn = !CMIN else cmn=5
 endif
 if n_elements(cmx) eq 0 then begin
   defsysv, '!CMAX', exists=i
   if i eq 1 then cmx = !CMAX else cmx=255
 endif
 
 if keyword_set(connect) then $
   if psym gt 0 then psym=-psym 

; Set up map coordinates and projection
 
 defsysv, '!POSVEC', exists=i
 if i eq 1 then poslast = !POSVEC

 defsysv, '!LIMIT', exists=i
 if i eq 1 then begin
   limits = !LIMIT
   if n_elements(minlat) eq 0 then minlat=limits(0)
   if n_elements(minlon) eq 0 then minlon=limits(1)
   if n_elements(maxlat) eq 0 then maxlat=limits(2)
   if n_elements(maxlon) eq 0 then maxlon=limits(3)
 endif
 if n_elements(region) eq 0 then begin
   if n_elements(minlat) eq 0 then minlat=-90
   if n_elements(maxlat) eq 0 then maxlat=90
   if n_elements(minlon) eq 0 then minlon=0
   if n_elements(maxlon) eq 0 then maxlon=360
 endif
 print,'minlat = ',minlat
 print,'maxlat = ',maxlat
 print,'minlon = ',minlon
 print,'maxlon = ',maxlon

 defsysv, '!PROJ', exists=i
 if i eq 1 then prj = !PROJ
 if prj eq 'sat' then begin
   defsysv, '!SUBLAT', exists=i
   if i eq 1 then sublat = !SUBLAT
   defsysv, '!SUBLNG', exists=i
   if i eq 1 then sublng = !SUBLNG
 endif
 
 case prj of 
  'cyl': map_set, 0.0, (minlon+maxlon)/2, 0, $
                  /cylindrical, limit=limits, $
                  /noerase, xmargin=[0,0], ymargin=[0,0], pos=poslast, /noborder
  'mer': map_set, 0.0, (minlon+maxlon)/2, 0, $
                  /mercator, limit=limits, $
                  /noerase, xmargin=[0,0], ymargin=[0,0], $
                  pos=poslast, /noborder
  'mol': map_set, 0.0, (minlon+maxlon)/2, 0, $
                  /mollweide, limit=limits, $
                  /noerase, xmargin=[0,0], ymargin=[0,0], $
                  pos=poslast, /noborder
  'sat': begin
           satparams=[6.61,0,0]  ; Note: 6.61 is GOES elev in globe radii
           map_set, sublat, sublng, /satellite, limit=limits, $
                    /noerase, xmargin=[0,0], ymargin=[0,0], $
                    pos=poslast, sat_p=satparams, /noborder
         end
  'polN': begin
            map_set, 90.0, (minlon+maxlon)/2.0, 0, $
                     /stereographic, limit=limits, $
                     /noerase, xmargin=[0,0], ymargin=[0,0], $
                     pos=poslast, /noborder, /label
          end
  'polS': begin
            map_set, -90.0, (minlon+maxlon)/2.0, 0, $
                     /stereographic, limit=limits, $
                     /noerase, xmargin=[0,0], ymargin=[0,0], $
                     pos=poslast, /noborder, /label
          end
  else: begin
            print, 'Requested projection not recognized' 
            stop         
        end
 endcase


; MULTI

 if keyword_set(multi) then begin
   !p.multi(0)=!p.multi(0)+1
 endif else begin
   !p.multi(0)=0
 endelse
 pmulti=!p.multi(0)







; Compute x and y position vectors for velovect 

 s=size(uarray)
 nlat=s[2]
 nlon=s[1]
 xarray = fltarr(nlon)
 yarray = fltarr(nlat)
 xsize=float(maxlon-minlon)/float(nlon)
 ysize=float(maxlat-minlat)/float(nlat)
 for i=0,nlon-1 do xarray[i]=minlon+xsize/2.0+xsize*i
 if keyword_set(order) then begin
   for i=0,nlat-1 do yarray[i]=maxlat-ysize/2.0-ysize*i
 endif else begin
   for i=0,nlat-1 do yarray[i]=minlat+ysize/2.0+ysize*i
 endelse

; Determine min and max data values 

 if (n_elements(vmn) EQ 0 OR n_elements(vmx) EQ 0) then begin
   if n_elements(missing) EQ 0 then begin
     dmin=9999.0
     dmax=0.0
     for i=0,nlon-1 do begin
       for j=0,nlat-1 do begin
         ws=sqrt(float(uarray[i,j])*float(uarray[i,j])+ $
               float(varray[i,j])*float(varray[i,j]))
         if (ws LT dmin) then dmin=ws
         if (ws GT dmax) then dmax=ws
       endfor
     endfor
   endif else begin
     m=where(uarray NE missing AND varray NE missing,cnt)
     dmin=9999.0
     dmax=0.0
     for i=0,cnt-1 do begin
       ws=sqrt(float(uarray[m[i]])*float(uarray[m[i]])+ $
               float(varray[m[i]])*float(varray[m[i]]))
       if (ws LT dmin) then dmin=ws
       if (ws GT dmax) then dmax=ws
     endfor
   endelse
 endif
 if (n_elements(vmn) EQ 0) then vmn=dmin
 if (n_elements(vmx) EQ 0) then vmx=dmax
 print,'vmin = ',vmn
 print,'vmax = ',vmx
 print,'cmin = ',cmn
 print,'cmax = ',cmx

; Draw velocity vectors over the top of the plot

 velovect2, uarray, varray, xarray, yarray, /noerase, /overplot, _extra=xtra, $
    cmin=cmn, cmax=cmx, vmin=vmn, vmax=vmx

; Draw the continental outlines and map grid

 if keyword_set(coast) then begin
   if keyword_set(usa) then begin
     map_continents, mlinethick=cth, /usa, /continents, color=ccol
   endif else begin
     if (coast EQ 2) then begin
       map_continents, mlinethick=cth, color=ccol, /coast, $
                       /fill_continents
     endif else begin
       map_continents, mlinethick=cth, color=ccol, /coast
     endelse
   endelse
 endif

; PNG?

 if keyword_set(pngout) AND !p.multi(0) LE 0 then begin
   image = tvrd()
   tvlct, /get, r, g, b
   print, gfile
   write_png, gfile, image, r, g, b
 endif

; MULTI?

 if keyword_set(multi) then begin
   !p.multi(0)=!p.multi(0)+1
 endif else begin
   !p.multi(0)=0
 endelse

end
