; $Id: //depot/Release/ENVI53_IDL85/idl/idldir/lib/fft_powerspectrum.pro#1 $
;
; Copyright (c) 1998-2015, Exelis Visual Information Solutions, Inc. All
;       rights reserved. Unauthorized reproduction is prohibited.
;
; From local dir: /usr/local/exelis53_85/idl85/lib
;
;+
;   FFT_POWERSPECTRUM
;
; PURPOSE:
;   Compute Fourier Power Spectrum
;
;
; CALLING SEQUENCE:
;
;   f = FFT_PPOWERSPECTRUM(x, dt, FREQ=freq, PERIOD=period)
;
;
; INPUTS:
;
;   x = original time series, of length N
;
;   dt = time interval between x measurements = Total time/N
;
;
; OUTPUT:
;
;   F = output power spectrum with N/2+1 components  (units are X^2)
;
;
; OPTIONAL KEYWORD INPUTS:
;
;   FRACTION = output F^2/TOTAL(F^2) (fractional power spectral density)
;
;   LAG = LAG 1 Autocorrelation, used for SIGNIF levels. Default is 0.0
;
;   SIG_LEVEL = significance level to use. Default is 0.05 (5%)
;
;   TUKEY = smooth F using a Tukey filter. Default is no smoothing.
;
;   WIDTH = the effective width of the Tukey filter, as a fraction of the
;           number of points N. Default is 0.02
;
;
; OPTIONAL KEYWORD OUTPUTS:
;
;   FREQ = output frequency components [ FREQ(n) = n/(Ndt) ]
;
;   PERIOD = output period components [ PERIOD(n) = 1/FREQ(n) ]
;
;   SIGNIFICANCE = output significance levels as a function of FREQ
;
;   AMP = Fourier amplitude
;
;   PHASE = Fourier phase
;
;   BANDWIDTH = The width of the smoothing filter in units of frequency
;
;   DOF = Degrees of freedom for chi-square distribution for significances.
;         For no smoothing this is 2.0.
;         For the Tukey it is 2*N*WIDTH
;
;
; MODIFICATION HISTORY:
;   Written C. Torrence
;   5 Aug 1998 (CT): added BOXCAR filter
;   3 Feb 1999 (CT): remove BOXCAR, added TUKEY
;   Aug 2014 (AGEH): updated for IDL 8.4
;-
;-------------------------------------------------------------

; Convolution of X with WGT, with zero-padding on the left,
; and reverse-X padding on the right.
; i.e. XPAD = [0,0,...,  x(0),x(1)...x(n-1),  x(n-1),x(n-2)...]
function FFTps_CONVOL, x, wgt
  compile_opt idl2, hidden

	n = x.length
	w = wgt.length
	x1 = [x, REVERSE([x[0:w/2], x[n-w/2-1:*]])]

	return, (CONVOL(x1, wgt, /EDGE_WRAP, /CENTER))[0:n-1]
	
end


function FFTps_FilterTukey, x, m, ndt, bandwidth, dof
  compile_opt idl2, hidden

	u = FINDGEN(2*m + 1) - m
	wgt = 0.5*(1 + COS(u*!PI/m))
	wgt = wgt/TOTAL(wgt)
	if (N_PARAMS() gt 2) then begin
		bandwidth = (4./3)*FLOAT(m)/(ndt)  ; Tukey bandwidth
		variance = (3./4)/FLOAT(m)         ; variance ratio
		dof = 2.*(4./3)*FLOAT(m)           ; degrees-of-freedom
	endif

	return, FFTps_CONVOL(x, wgt)
	
end

function FFT_PowerSpectrum, x, dtIn, $
    FRACTION=fraction, LAG=lagIn, SIG_LEVEL=siglvlIn, $
    TUKEY=tukey, WIDTH=widthIn, FREQ=freq, PERIOD=period, $
    AMP=amp, PHASE=phase, FFT_THEOR=fft_theor, $
    SIGNIFICANCE=signif, DOF=dof, BANDWIDTH=bandwidth

  compile_opt idl2, hidden
  ON_ERROR,2

  N = N_ELEMENTS(x)
  if (N eq 0) then begin
    message, 'Invalid input'
    return, 0
  endif

  dt = N_ELEMENTS(dtIn) eq 0 ? 1.0 : dtIn[0]
  siglvl = N_ELEMENTS(siglvlIn) eq 0 ? 0.05 : siglvlIn[0]
  lag = N_ELEMENTS(lagIn) eq 0 ? 0.0 : lagIn[0]
  width = N_ELEMENTS(widthIn) eq 0 ? 0.02 : widthIn[0]

  fft_x = FFT( x[*] , -1)
  amp = 2*ABS(fft_x[0:(N+1)/2-1])

  if ((N mod 2) eq 0) then begin  ;Nyquist is 1/2 power for N even
    amp = [amp, SQRT(2)*ABS(fft_x[N/2])]
  endif
  phase = (ATAN(IMAGINARY(fft_x), FLOAT(fft_x)))[0:(N+1)/2-1]
  power_spec = 0.5*amp^2
  variance = TOTAL(power_spec[1:*])

  fraction = KEYWORD_SET(fraction) ? 1./variance : 1

  nf = N/2 + 1
  freq = FINDGEN(nf)/(N*dt)
  period = [N*dt, 1./freq[1:*]]

  dof = 2.
  if (KEYWORD_SET(tukey)) then begin
    m = (3./4)*width*N
    power_spec = FFTps_FilterTukey(power_spec, m, N*dt, bandwidth, dof)
  endif

  fft_theor = (1 - lag^2)/(1 - 2*lag*COS(dt*freq*2*!PI) + lag^2)
  fft_theor *= variance*2./N*fraction
  signif = fft_theor*(CHISQR_CVF(1.-siglvl, FLOAT(dof))/FLOAT(dof))

  return, power_spec*fraction
  
end

;+
; NAME:
;  cgGREEK
;
; PURPOSE:
;
;   This function provides a device-independent way to ask for a Greek letter as
;   a string that can be included, for example, in a plot title. It uses the Greek
;   simplex font (!4) when used with Hershey fonts, and the Symbol font (!9) when
;   used with PostScript or True-Type fonts. Selects the type of Greek character to 
;   return based on value of !P.FONT. Updated now to return the UNICODE values for 
;   Greek characters for those fonts that support them (Macintosh?).
;   
; AUTHOR:
;
;   FANNING SOFTWARE CONSULTING
;   David Fanning, Ph.D.
;   1645 Sheely Drive
;   Fort Collins, CO 80526 USA
;   Phone: 970-221-0438
;   E-mail: david@idlcoyote.com
;   Coyote's Guide to IDL Programming: http://www.idlcoyote.com/
;
; CATEGORY:
;
;   Graphics, Utilities
;
; CALLING SEQUENCE:
;
;   greekString = cgGreek(greekLetter)
;
; RETURN VALUE:
;
;   greekString    A string that represents the Greek letter.
;
; ARGUMENTS:
;
;  greekLetter:    The name of the Greek letter desired. A string. Default: 'alpha'.
;                  Valid string names are the 24 characters of the Greek alphabet.
;                     alpha        nu
;                     beta         xi
;                     gamma        omicron
;                     delta        pi
;                     epsilon      rho
;                     zeta         sigma
;                     eta          tau
;                     theta        upsilon
;                     iota         phi
;                     kappa        chi
;                     lambda       psi
;                     mu           omega
;                    
;                   Note that if the first letter of the name is capitalized, this is
;                   the equivalent of setting the CAPITAL keyword. 
;
; KEYWORDRS:
;
;  CAPTIAL:        If this keyword is set, the captial Greek letter is returned rather 
;                  than the lowercase Greek letter. An alternative way of capitalizing
;                  the letter is to make the first letter of the name an uppercase letter.
;                  
;  EXAMPLE:        If this keyword is set, the names of the Greek characters and their
;                  symbols are written out in the current graphics window.
;                  
;  PS:             Normally, the PostScript version of the greek letter is returned if
;                  the current device is PostScript and !P.Font is 0 or 1. But, the 
;                  PostScript version of the greek letter can be obtained at any time
;                  and in any device, by setting this keyword.
;                                    
;  UNICODE:        If this keyword is set, the function returns the Unicode for the Greek
;                  letter.
;
; EXAMPLE:
;
;  Lowercase PSI:
;  
;     IDL> Plot, findgen(11), XTitle='This title contains ' + $
;           cgGreek('psi') + ' as a Greek letter' 
;
;  Uppercase PSI:
;  
;     IDL> Plot, findgen(11), XTitle='This title contains ' + $
;           cgGreek('Psi') + ' as a Greek letter' 
; NOTES:
; 
;  See the following article for additional information: 
;  
;       http://www.idlcoyote.com/ps_tips/greeksym.html
;       
; RESTRICTIONS:
; 
;  For this program to work correctly on your graphics display, you should be using
;  Hershey fonts (!P.Font=-1). It will work correctly in PostScript with either 
;  hardware fonts (!P.Font=0) or True-Type fonts (!P.Font=1).
;  
; MODIFICATION HISTORY:
;
;  Written by: David W. Fanning, 9 January 2010.
;  An alternative way to get an uppercase letter is to make the first letter of
;     the Greek name uppercase. (Maarten Sneep's suggestion!) 11 Jan 2010. DWF
;  Had the wrong value for the PostScript version of Phi. 26 January 2010. DWF
;  Added UNICODE keyword and values for Greek characters. 11 June 2010. DWF.
;  Changed the branching from !D.NAME EQ 'PS' to !P.FONT NE -1. (This is actually
;      what the documentation says, and what I intended.) 13 Dec 2010. DWF.
;  I don't think the last change did quite want I wanted. More tweaking to make
;      this more responsive to being in a PostScript file. 31 July 2011. DWF.
;  Added PS keyword so the return value is the PostScript file. This is for
;      convenience only, as the return value will be the PostScript value if
;      the current graphics device is PS and FONT is not equal to -1. 30 Aug 2011. DWF.
;  Retired Greek and replaced by cgGreek. 23 Dec 2012. DWF.
;-
;******************************************************************************************;
;  Copyright (c) 2010, by Fanning Software Consulting, Inc.                                ;
;  All rights reserved.                                                                    ;
;                                                                                          ;
;  Redistribution and use in source and binary forms, with or without                      ;
;  modification, are permitted provided that the following conditions are met:             ;
;                                                                                          ;
;      * Redistributions of source code must retain the above copyright                    ;
;        notice, this list of conditions and the following disclaimer.                     ;
;      * Redistributions in binary form must reproduce the above copyright                 ;
;        notice, this list of conditions and the following disclaimer in the               ;
;        documentation and/or other materials provided with the distribution.              ;
;      * Neither the name of Fanning Software Consulting, Inc. nor the names of its        ;
;        contributors may be used to endorse or promote products derived from this         ;
;        software without specific prior written permission.                               ;
;                                                                                          ;
;  THIS SOFTWARE IS PROVIDED BY FANNING SOFTWARE CONSULTING, INC. ''AS IS'' AND ANY        ;
;  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES    ;
;  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT     ;
;  SHALL FANNING SOFTWARE CONSULTING, INC. BE LIABLE FOR ANY DIRECT, INDIRECT,             ;
;  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED    ;
;  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;         ;
;  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND             ;
;  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT              ;
;  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS           ;
;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                            ;
;******************************************************************************************;
Forward_Function cgGreek

PRO cgGreek_Example, UNICODE=unicode, PS=ps

    Compile_Opt hidden
    
    ; The 24 Greek letters.
    letter = [  'alpha', 'beta', 'gamma', 'delta', 'epsilon',  'zeta', $
                'eta', 'theta', 'iota', 'kappa', 'lambda', 'mu', $
                'nu', 'xi', 'omicron', 'pi', 'rho', 'sigma', 'tau', $
                'upsilon', 'phi', 'chi', 'psi', 'omega' ]
    
    ; Output positions.
    x = [0.25, 0.6]
    y = Reverse((Indgen(12) + 1) * (1.0 / 13))
    
    ; Need PostScript output?
    IF Keyword_Set(ps) || (!D.Name EQ 'PS') THEN cgPS_Open
    
    ; Create a window.
    cgDisplay, 600, 500
    
    ; Output the letters.
    FOR j=0,11 DO BEGIN
        cgText, x[0], y[j], letter[j] + ': ' + $
            cgGreek(letter[j], UNICODE=unicode) + cgGreek(letter[j], /CAPITAL, UNICODE=unicode), $
            /NORMAL, CHARSIZE=1.5
        cgText, x[1], y[j], letter[j+12] + ': ' + $
            cgGreek(letter[j+12], UNICODE=unicode) + cgGreek(letter[j+12], /CAPITAL, UNICODE=unicode), $
            /NORMAL, CHARSIZE=1.5
    ENDFOR
    
    ; Clean up PostScript, if needed.
    IF Keyword_Set(ps) || (!D.Name EQ 'PS') THEN cgPS_Close

    ; Restore the users window.
    IF N_Elements(thisWindow) NE 0 THEN BEGIN
       IF thisWindow GE 0 THEN WSet, thisWindow
    ENDIF
    
END ; --------------------------------------------------------------------------------------


FUNCTION cgGreek, letter, CAPITAL=capital, EXAMPLE=example, PS=PS, UNICODE=unicode

    Compile_Opt idl2
    
    ; Return to caller on error.
    ON_Error, 2
    
    ; Do you wish to see an example?
    IF Keyword_Set(example) THEN BEGIN
        cgGreek_Example, UNICODE=unicode, PS=ps
        RETURN, ""
    ENDIF

    ; A symbol name is required.
    IF N_Elements(letter) EQ 0 THEN BEGIN
       Print, 'Syntax: Greek("theLetter")
       RETURN, ""
    ENDIF
    
    ; Return quietly if the Greek letter is a null string.
    IF letter  EQ "" THEN RETURN, ""
    
     ; Set up PostScript device for working with Greek letters.
    IF !D.Name EQ 'PS' THEN Device, ISOLATIN1=1
    
    ; Check keywords.
    IF N_Elements(letter) EQ 0 THEN letter = 'alpha'
    capital = Keyword_Set(capital)
    
    ; If the first letter of the "letter" variable is uppercase
    ; the user wants an uppercase Greek letter.
    firstLetter = StrMid(letter, 0, 1)
    IF firstLetter EQ StrUpCase(firstLetter) THEN capital = 1
    
    IF Keyword_Set(unicode) THEN BEGIN
    
        CASE StrLowCase(letter) OF
            'alpha':   greekLetter = (capital) ? '!Z(0391)' : '!Z(03B1)'
            'beta':    greekLetter = (capital) ? '!Z(0392)' : '!Z(03B2)'
            'gamma':   greekLetter = (capital) ? '!Z(0393)' : '!Z(03B3)'
            'delta':   greekLetter = (capital) ? '!Z(0394)' : '!Z(03B4)'
            'epsilon': greekLetter = (capital) ? '!Z(0395)' : '!Z(03B5)'
            'zeta':    greekLetter = (capital) ? '!Z(0396)' : '!Z(03B6)'
            'eta':     greekLetter = (capital) ? '!Z(0397)' : '!Z(03B7)'
            'theta':   greekLetter = (capital) ? '!Z(0398)' : '!Z(03B8)'
            'iota':    greekLetter = (capital) ? '!Z(0399)' : '!Z(03B9)'
            'kappa':   greekLetter = (capital) ? '!Z(039A)' : '!Z(03BA)'
            'lambda':  greekLetter = (capital) ? '!Z(039B)' : '!Z(03BB)'
            'mu':      greekLetter = (capital) ? '!Z(039C)' : '!Z(03BC)'
            'nu':      greekLetter = (capital) ? '!Z(039D)' : '!Z(03BD)'
            'xi':      greekLetter = (capital) ? '!Z(039E)' : '!Z(03BE)'
            'omicron': greekLetter = (capital) ? '!Z(039F)' : '!Z(03BF)'
            'pi':      greekLetter = (capital) ? '!Z(03A0)' : '!Z(03C0)'
            'rho':     greekLetter = (capital) ? '!Z(03A1)' : '!Z(03C1)'
            'sigma':   greekLetter = (capital) ? '!Z(03A3)' : '!Z(03C3)'
            'tau':     greekLetter = (capital) ? '!Z(03A4)' : '!Z(03C4)'
            'upsilon': greekLetter = (capital) ? '!Z(03A5)' : '!Z(03C5)'
            'phi':     greekLetter = (capital) ? '!Z(03A6)' : '!Z(03C6)'
            'chi':     greekLetter = (capital) ? '!Z(03A7)' : '!Z(03C7)'
            'psi':     greekLetter = (capital) ? '!Z(03A8)' : '!Z(03C8)'
            'omega':   greekLetter = (capital) ? '!Z(03A9)' : '!Z(03C9)'
            ELSE: Message, 'The greek letter ' + letter + ' is unrecognized.'
       ENDCASE
           
       ; Return the UNICODE greek letter.
       RETURN, greekLetter
       
    ENDIF
    
    IF ((!D.Name EQ 'PS') && (!P.FONT NE -1)) || Keyword_Set(PS)  THEN BEGIN
    
        ; Make sure ISOLATIN1 encoding is turned on.
        IF (!D.Name EQ 'PS') THEN DEVICE, /ISOLATIN1
        
        CASE StrLowCase(letter) OF
            'alpha':   greekLetter = (capital) ? '!9' + String("101B) + '!X' : '!9' + String("141B) + '!X'
            'beta':    greekLetter = (capital) ? '!9' + String("102B) + '!X' : '!9' + String("142B) + '!X'
            'gamma':   greekLetter = (capital) ? '!9' + String("107B) + '!X' : '!9' + String("147B) + '!X'
            'delta':   greekLetter = (capital) ? '!9' + String("104B) + '!X' : '!9' + String("144B) + '!X'
            'epsilon': greekLetter = (capital) ? '!9' + String("105B) + '!X' : '!9' + String("145B) + '!X'
            'zeta':    greekLetter = (capital) ? '!9' + String("132B) + '!X' : '!9' + String("172B) + '!X'
            'eta':     greekLetter = (capital) ? '!9' + String("110B) + '!X' : '!9' + String("150B) + '!X'
            'theta':   greekLetter = (capital) ? '!9' + String("121B) + '!X' : '!9' + String("161B) + '!X'
            'iota':    greekLetter = (capital) ? '!9' + String("111B) + '!X' : '!9' + String("151B) + '!X'
            'kappa':   greekLetter = (capital) ? '!9' + String("113B) + '!X' : '!9' + String("153B) + '!X'
            'lambda':  greekLetter = (capital) ? '!9' + String("114B) + '!X' : '!9' + String("154B) + '!X'
            'mu':      greekLetter = (capital) ? '!9' + String("115B) + '!X' : '!9' + String("155B) + '!X'
            'nu':      greekLetter = (capital) ? '!9' + String("116B) + '!X' : '!9' + String("156B) + '!X'
            'xi':      greekLetter = (capital) ? '!9' + String("130B) + '!X' : '!9' + String("170B) + '!X'
            'omicron': greekLetter = (capital) ? '!9' + String("117B) + '!X' : '!9' + String("157B) + '!X'
            'pi':      greekLetter = (capital) ? '!9' + String("120B) + '!X' : '!9' + String("160B) + '!X'
            'rho':     greekLetter = (capital) ? '!9' + String("122B) + '!X' : '!9' + String("162B) + '!X'
            'sigma':   greekLetter = (capital) ? '!9' + String("123B) + '!X' : '!9' + String("163B) + '!X'
            'tau':     greekLetter = (capital) ? '!9' + String("124B) + '!X' : '!9' + String("164B) + '!X'
            'upsilon': greekLetter = (capital) ? '!9' + String("125B) + '!X' : '!9' + String("165B) + '!X'
            'phi':     greekLetter = (capital) ? '!9' + String("106B) + '!X' : '!9' + String("152B) + '!X'
            'chi':     greekLetter = (capital) ? '!9' + String("103B) + '!X' : '!9' + String("143B) + '!X'
            'psi':     greekLetter = (capital) ? '!9' + String("131B) + '!X' : '!9' + String("171B) + '!X'
            'omega':   greekLetter = (capital) ? '!9' + String("127B) + '!X' : '!9' + String("167B) + '!X'
            ELSE: Message, 'The greek letter ' + letter + ' is unrecognized.'
        ENDCASE
    
    ENDIF ELSE BEGIN
    
        CASE StrLowCase(letter) OF
            'alpha':   greekLetter = (capital) ? '!4' + String("101B) + '!X' : '!4' + String("141B) + '!X'
            'beta':    greekLetter = (capital) ? '!4' + String("102B) + '!X' : '!4' + String("142B) + '!X'
            'gamma':   greekLetter = (capital) ? '!4' + String("103B) + '!X' : '!4' + String("143B) + '!X'
            'delta':   greekLetter = (capital) ? '!4' + String("104B) + '!X' : '!4' + String("144B) + '!X'
            'epsilon': greekLetter = (capital) ? '!4' + String("105B) + '!X' : '!4' + String("145B) + '!X'
            'zeta':    greekLetter = (capital) ? '!4' + String("106B) + '!X' : '!4' + String("146B) + '!X'
            'eta':     greekLetter = (capital) ? '!4' + String("107B) + '!X' : '!4' + String("147B) + '!X'
            'theta':   greekLetter = (capital) ? '!4' + String("110B) + '!X' : '!4' + String("150B) + '!X'
            'iota':    greekLetter = (capital) ? '!4' + String("111B) + '!X' : '!4' + String("151B) + '!X'
            'kappa':   greekLetter = (capital) ? '!4' + String("112B) + '!X' : '!4' + String("152B) + '!X'
            'lambda':  greekLetter = (capital) ? '!4' + String("113B) + '!X' : '!4' + String("153B) + '!X'
            'mu':      greekLetter = (capital) ? '!4' + String("114B) + '!X' : '!4' + String("154B) + '!X'
            'nu':      greekLetter = (capital) ? '!4' + String("115B) + '!X' : '!4' + String("155B) + '!X'
            'xi':      greekLetter = (capital) ? '!4' + String("116B) + '!X' : '!4' + String("156B) + '!X'
            'omicron': greekLetter = (capital) ? '!4' + String("117B) + '!X' : '!4' + String("157B) + '!X'
            'pi':      greekLetter = (capital) ? '!4' + String("120B) + '!X' : '!4' + String("160B) + '!X'
            'rho':     greekLetter = (capital) ? '!4' + String("121B) + '!X' : '!4' + String("161B) + '!X'
            'sigma':   greekLetter = (capital) ? '!4' + String("122B) + '!X' : '!4' + String("162B) + '!X'
            'tau':     greekLetter = (capital) ? '!4' + String("123B) + '!X' : '!4' + String("163B) + '!X'
            'upsilon': greekLetter = (capital) ? '!4' + String("124B) + '!X' : '!4' + String("164B) + '!X'
            'phi':     greekLetter = (capital) ? '!4' + String("125B) + '!X' : '!4' + String("165B) + '!X'
            'chi':     greekLetter = (capital) ? '!4' + String("126B) + '!X' : '!4' + String("166B) + '!X'
            'psi':     greekLetter = (capital) ? '!4' + String("127B) + '!X' : '!4' + String("167B) + '!X'
            'omega':   greekLetter = (capital) ? '!4' + String("130B) + '!X' : '!4' + String("170B) + '!X'
            ELSE: Message, 'The greek letter ' + letter + ' is unrecognized.'      
        ENDCASE
        
      ENDELSE

    RETURN, greekLetter
    
END

:******** Begin Main program


@transect_A2013216201500.pro
@fft_powerspectrum_vSC.pro

!x.thick = 3
!y.thick = 3
!p.thick = 2
;!p.charsize=1.3
!p.font = 0


; Define solid circles for symbols
a=findgen(16)*(!pi/8.)
usersym,cos(a),sin(a),/fill,thick=3
symsize=1.3

openr,in,'colors_25.tbl',/get_lun
colarray=intarr(3,27)
readf,in,colarray
close,in
free_lun,in
red = colarray(0,*)
green = colarray(1,*)
blue = colarray(2,*)
tvlct, red, green, blue
;colall=[0.5,3.5,5.8,7.0,9.5,10.5,12.8,14.5,15.5,17.5,20,22.5,8,24,25]
;colall=[0.5,3.5, 7.0, 9.5, 10.5, 12.8, 15.5, 17.5, 20, 22.5, 8,24,25]
colall=[10.5,0.5,3.5, 7.0, 9.5, 12.8, 15.5, 17.5, 20, 22.5, 8,24,25]


lonmin = -150.05
lonmax = -149.65
latmax = 72.0
latmin = 70.3

modfreq = [ 0.00000,0.0714286,0.142857,0.214286,0.285714,0.357143,0.428571,0.500000]
modpwr = [7.51802,0.798293,0.185960,0.102009,0.0450783,0.0489643,0.0393479,0.0169167] ;6

mod_0_11_fbin = [0.00000,0.0555556,0.111111,0.166667,0.222222,0.277778,0.333333,0.388889,0.444444,0.500000,0.555556]
mod_0_11_bin = [3.88999,1.71274,0.372073,0.192587,0.145880,0.0331663,0.0376632,0.0463815,0.0448916,0.0261585]
mod_1_10_bin = [4.23301,1.55072,0.603913,0.172293,0.115285,0.0465434,0.0363101,0.0218991,0.0210885,0.00976911]
mod_2_9_bin = [4.86714,1.61792,0.464860,0.118508,0.0558786,0.0387758,0.0288929,0.0180840,0.0258560,0.0115054]
mod_2_10_bin = [4.69687,1.57100,0.511038,0.134529,0.0642706,0.0387758,0.0288929,0.0180840,0.0258560,0.0115054]
mod_3_8_bin = [5.03445,1.68103,0.272716,0.134820,0.0736805,0.0301514,0.0230890,0.0181798,0.0231955,0.0105448 ]
modpwravg = [5.03445,1.51902,0.217485,0.0588495,0.0385699,0.0186556,0.0161703,0.0186450,0.0120669,0.0109621]

modir = 'matchups/besst/'
;modlist = modir+'*.sst_besst_allq.txt'  ;as above but has qc column -  SINGLE GRANULE

;;; MODIS GRANULES
;modname = 'T2013216195500'
modname = 'A2013216201500'  ;11:15 LST
;modname = 'T2013216213000'  ;not as bad - similar to the Aqua above - first 2 legs are bad
;modname = 'A2013216215000'  ;very bad
modlist = modir+modname+'.sst_besst_allq.txt'  ;as above but has MODIS qc column - ALL GRANULEs

modcol = 4

spawn,('ls -1 '+modlist),mod_file
print,mod_file
nfiles = n_elements(mod_file)
;stop

for x=0,nfiles-1 do begin

  dum = str_sep(mod_file(x),'/')
  dum2 = str_sep(dum(2),'.')
  modfile = dum2(0)
  modfilest = strcompress(modfile,/remove_all)

  ml = file_lines(mod_file(x))
  modis = fltarr(modcol, ml)  ;qc colum new
  openr,unit,mod_file(x),/get_lun
  readf,unit,modis
  close,unit
  free_lun,unit

  okm = where(modis(1,*) ge lonmin and modis(1,*) le lonmax and modis(0,*) ge latmin and modis(0,*) le latmax and modis(2,*) ge -1.98,cokm)
  modis = modis(*,okm)
  mod_lat = reform(modis(0,*))
  mod_lon = reform(modis(1,*))
  mod_sst = modis(2,*)
  mod_qc = modis(3,*)

  legidx=transect_A2013216201500(mod_lon)
  PLOTSYM, 3,1.2,/fill

  ;**** Paper PLOT
  ;window,1
  set_plot,'ps'
  ;device,/encapsulated,filename='plots/A2013216201500_scanlines.eps',xsize=8.5,ysize=6.0,/inches,/color
  device,/encapsulated,filename='plots/paper/castro_fig6a.eps',xsize=8.5,ysize=6.0,/inches,/color,$
   font_size=15,yoffset=1.5,xoffset=1,bit=8,set_font='times-Bold'

  plot,mod_lon,mod_lat,psym=2,/ynoz,xtitle='MODIS Longitude',ytitle='MODIS Latitude',title=modfilest,$
   xrange=[-150.03,-149.65],xstyle=1,/nodata
  ;*** Plots ALL the tracks
  ;**** next line plots them all; for paper use oplot inside the loop for the ones in the final analysis
  ;for i=0,cokm-1 do plots,mod_lon(i),mod_lat(i),psym=2,color=colall(legidx(i))
  ;device,/close
  ;stop

  modspectra = replicate(!values.f_nan,13,10)
  modfreqarr = replicate(!values.f_nan,13,10)

  ;modis tracks: 2 -11
  for j=2,11 do begin

    trackid = where(legidx eq j,cj)
    trackst = strcompress(string(j),/remove_all)
    slat = mod_lat(trackid)
    slon = mod_lon(trackid)
    ssst = mod_sst(trackid)

    srtlon = sort(slon)
    ssst = ssst(srtlon)
    slons = slon(srtlon)
    slats = slat(srtlon)

    ;* interpolate gaps 
    if j eq 8 then begin
      newsst = fltarr(cj+1)
      newsst(0:2) = ssst(0:2)
      newsst(3) = 0.5*(ssst(2)+ssst(3))+ssst(2)
      newsst(4:cj) = ssst(3:cj-1)
      ssst=newsst
      ;stop
    endif

    spec = FFT_PowerSpectrum(ssst,1, FREQ=freq, significance=signif,sig_level=0.025) ;km
    ;spec = FFT_PowerSpectrum(ssst,1, FREQ=freq, significance=signif) ;km
    ;print,freq
    ;print,signif(0)

    ;*** What is the other end of the significance level
    N = n_elements(ssst)
    siglvl=0.025
    signif_orig = float(signif(0))/(CHISQR_CVF(0.975, 2.0)/2.0)
    sig_ll = signif_orig*(CHISQR_CVF(siglvl, FLOAT(N*2.0))/FLOAT(N*2.0))^(-1)
    sig_ul = signif_orig*(CHISQR_CVF(1.-siglvl, FLOAT(N*2.0))/FLOAT(N*2.0))^(-1)
    print,'LL Significance: ',sig_ll
    print,'UL Significance: ',sig_ul
    ;stop

    ;window,2
    ;plot,freq,spec,/ylog,psym=-2,/noerase,$
    ;  XTITLE='Wavenumber (1/km)',title= 'MODIS Power Reduced Spectrum for Track No. '+trackst
    ;oplot,freq,signif,color=25

    ns=size(spec,/n_elements)
    modspectra(0:ns-1,j-2)=spec
    modfreqarr(0:ns-1,j-2)=freq
    
    ;!!paper plot: plot tracks used in the analysis only
    if j ge 3 then oplot,slon,slat,psym=8,color=colall(j)

  endfor   
  xyouts,-150.02,71.97,'(a)',charsize=1.5
  device,/close
  ;stop

  ;* rejecting first track  (remove this or not???)
  modspectra = modspectra(*,1:9)

  ;** Bin array
  ;binsize = 0.05
  ;binsize = 0.0555556  ;terra  : 18 km
  binsize = 0.0416667  ;aqua  : 24 km  -- used
  ;binsize = 0.0714286   ;original terra 14 km (bin aqua and leave terra un binned at the orig res)


  mbinid =  modfreqarr/binsize
  ok = where(finite(mbinid))
  mbinid(ok) = round( mbinid(ok))

  ;*** Average by column
  modspecbin = fltarr(13)
  frqbin=indgen(13)*binsize
  for i=0,12 do begin
      idbin = where(mbinid eq i,cid)
      modspecbin(i) = mean(modspectra(idbin),/NaN)
      ;modspecbin(i) = mean(modspectrared(idbin),/NaN)
  endfor
  ;stop

  ;binning by 0.04 (aqua)
  test=fltarr(2,13)
  test(0,*) = modspecbin

  ;*** binning terra (It seems that the original is better behaved)
  terra_bin = replicate(!values.f_nan,13)

  ;*** using the original Terra
  tbinid =  modfreq/binsize              ;original
  tbinid = round(tbinid)
  terra_bin(tbinid)=modpwr              ;original

  ;*** using the binned Terra
  ;tbinid =  mod_0_11_fbin(0:9)/binsize  ;new
  ;tbinid = round(tbinid)
  ;terra_bin(tbinid)=mod_2_10_bin       ;new
  ;terra_bin(tbinid)=mod_3_8_bin       ;new
  ;terra_bin(tbinid)=modpwravg       ;avg terra de 3-8

  test(1,*) = terra_bin
  tstm = mean(test,dimension=1,/Nan)    ;mean terra-aqua

  ;binning by 0.0555 (terra)
  ;test=fltarr(2,10)
  ;test(0,*) = modspecbin(0:9)
  ;test(1,*) = mod_0_11_bin
  ;test(1,*) = mod_1_10_bin
  ;test(1,*) = mod_2_9_bin
  ;test(1,*) = mod_2_10_bin
  ;tstm = mean(test,dimension=1,/Nan)

  ;specavg = mean(modspectrared,dimension=2,/Nan)
  specavg = mean(modspectra,dimension=2,/Nan)
  freqavg = modfreqarr(*,1)
  ;print,'Avg AQUA: 'specavg
  ;print'Avg AQUA+TERRA: ',tstm
  ;stop

  items = ['AQUA','TERRA','Average']
  PLOTSYM,0,1.2,/fill
  symarr=[-2,-4,88]
  linearr=[0,0,1]
  thickl=replicate(5,4)
  ;OJO! when using cgGrek in legend need to unse /normal!!!!!!!!!!!!
  nustr = cgGreek('nu')+'!U-2!N'
  items_loglog = ['AQUA','TERRA','Average',nustr]

  ;window,0
  ;** paper
  ;set_plot,'ps'
  ;filename='plots/modis_spectra_semilog_2.eps'
  ;filename='plots/modis_spectra_semilog_3.eps'  ;with all scan lines in Aqua
  filename='plots/paper/castro_fig7a.eps' ;  modis_spectra_semilog_3.eps' all lines in Aqua
  device,/encapsulated,filename=filename,xsize=8.5,ysize=6.0,/inches,$
      font_size=15,yoffset=1.5,xoffset=1,set_font='times-Bold',bit=8;
  ;proplotsimple,filename,font_size=4.0,xsize=8.5,ysize=6.0
  ;muletter = "155B
  ;mustr = string(muletter)
  plot,frqbin,modspecbin,/ylog,psym=-2,thick=3.0,symsize=1.5,$
     title='MODIS Wavenumber Spectra',ytitle='SST Power [(!Uo!NC)!U2!N km]',XTITLE=cgGreek('nu')+' [km!U-1!N]'
     ;title='MODIS Wavenumber Spectra',ytitle='SST Power [(!Uo!NC)!U2!N km]',XTITLE='!7m!x [km!U-1!N]'
     ;title='MODIS Wavenumber Spectra',ytitle='SST Power [(!Uo!NC)!U2!N km]',XTITLE='!4'+mustr+'!X [km!U-1!N]',font=-1
  oplot,modfreq,modpwr,psym=-4,thick=3,symsize=1.5
  oplot,frqbin,tstm,psym=8;,symsize=3.0;,thick=2
  oplot,frqbin,tstm,linestyle=1,thick=3
  ;oplot,mod_0_11_fbin(0:9),mod_3_8_bin,psym=-4,color=8
  ;oplot,mod_0_11_fbin(0:9),modpwravg,psym=-4,color=8
  legend,items,psym=symarr,number=2,linestyle=linearr,/top,/right,/fill,box=1,spacing=1.5,thick=thickl(0:2)
  xyouts,0.012,60,'(a)',charsize=1.5
  device,/close
  ;stop

  ;window,2
  symarr=[-2,-4,88,0]
  linearr=[0,0,1,4]
  ;filename_ll='plots/modis_spectra_loglog_2.eps'
  ;filename_ll='plots/modis_spectra_loglog_3.eps'
  filename_ll='plots/paper/castro_fig7b.eps'
  ;proplotsimple,filename_ll,font_size=4.0,xsize=8.5,ysize=6.0
  device,/encapsulated,filename=filename_ll,xsize=8.5,ysize=6.0,/inches,$
      font_size=15,yoffset=1.5,xoffset=1,set_font='times-Bold',bit=8;
  plot,frqbin,modspecbin,/ylog,/xlog,xrange=[0.01,1],xstyle=1,psym=-2,thick=3.0,symsize=1.5,$
     title='MODIS Wavenumber Spectra',ytitle='SST Power [(!Uo!NC)!U2!N km]',XTITLE=cgGreek('nu')+' [km!U-1!N]'
     ;title='MODIS Wavenumber Spectra',ytitle='SST Power [(!Uo!NC)!U2!N km]',XTITLE='!7m!x [km!U-1!N]'
     ;title='MODIS Wavenumber Spectra',ytitle='SST Power [(!Uo!NC)!U2!N km]',XTITLE='!4'+mustr+'!X [km!U-1!N]',font=-1
  oplot,modfreq,modpwr,psym=-4,thick=3,symsize=1.5
  oplot,frqbin,tstm,psym=8;,symsize=3.0;,thick=2
  oplot,frqbin,tstm,linestyle=1,thick=3
  oplot,frqbin,0.003*frqbin^(-2),linestyle=4,thick=3
  ;oplot,frqbin,0.0001*frqbin^(-3),linestyle=3,thick=2
  ;oplot,mod_0_11_fbin(0:9),mod_3_8_bin,psym=-4,color=8
  ;oplot,mod_0_11_fbin(0:9),modpwravg,psym=-4,color=8
  ;items_loglog = ['AQUA','TERRA','Average','!4'+mustr+'!X!U-2!N']
  legend,items_loglog,psym=symarr,number=2,linestyle=linearr,/top,/right,/fill,box=1,spacing=1.5,/normal,thick=thickl
  xyouts,0.011,7,'(b)',charsize=1.5
  device,/close
  stop

  set_plot,'x'
  ;window,0
  plot,frqbin,modspecbin,/ylog,XTITLE='Wavenumber (1/km)',title= 'MODIS Power Spectrum',psym=-2
  oplot,modfreq,modpwr,psym=-2,color=11
  ;oplot,mod_0_11_fbin,mod_0_11_bin,psym=-4,color=20
  ;oplot,mod_0_11_fbin(0:9),mod_1_10_bin,psym=-4,color=15
  ;oplot,mod_0_11_fbin(0:9),mod_2_9_bin,psym=-4,color=8
  ;oplot,mod_0_11_fbin(0:9),mod_2_10_bin,psym=-4,color=5
  oplot,frqbin,tstm,psym=-4,color=25

  ;window,2
  plot,frqbin,modspecbin,/ylog,/xlog,xrange=[0.01,1],xstyle=1,$
     XTITLE='Wavenumber (1/km)',title= 'MODIS Power Spectrum',psym=-2
  oplot,modfreq,modpwr,psym=-2,color=11
  ;oplot,mod_0_11_fbin,mod_0_11_bin,psym=-4,color=20
  ;oplot,mod_0_11_fbin(0:9),mod_1_10_bin,psym=-4,color=15
  ;oplot,mod_0_11_fbin(0:9),mod_2_9_bin,psym=-4,color=8
  ;oplot,mod_0_11_fbin(0:9),mod_2_10_bin,psym=-4,color=5

  ;** binning by 0.0555
  ;oplot,mod_0_11_fbin(0:9),tstm,psym=-4,color=25

  ;** binning by 0.04
  oplot,frqbin,tstm,psym=-4,color=25
  stop

endfor

stop
end

;**** Main code for computing the BESST Spectra

@fft_powerspectrum_vSC.pro
@cgGreek.pro
!path = '~/idl_library:' + !path  

set_plot,'ps'

!p.thick = 2
!x.thick = 3
!y.thick = 3
!p.font = 0
!p.charsize=1.2

red   = [0,1,1,0,0,1,0]
green = [0,1,0,1,0,0,1]
blue  = [0,1,0,0,1,1,1]
tvlct, 255*red, 255*green, 255*blue

;*** original terra binsize is 14 km
modfreq = [ 0.00000,0.0714286,0.142857,0.214286,0.285714,0.357143,0.428571,0.500000]
;modpwr = [8.56793,0.934411,0.219232,0.119300,0.0532247,0.0533752,0.0461836,0.0200450] ;5
modpwr = [7.51802,0.798293,0.185960,0.102009,0.0450783,0.0489643,0.0393479,0.0169167] ;6

;*** this freq applies to all the ones below (binsize 0.0416667)
modfreq_A = [ 0.00000,0.0416667,0.0833334,0.125000,0.166667,0.208334,0.250000,0.291667,0.333334,0.375000,0.416667,0.458334,0.500000]

;*** removing the very first line of the modis spectra and binning freq to 24 km
modpwr_A = [22.2539,3.15573,1.16935,0.421450,0.182096,0.155993,0.0851479,0.0920441,0.110567,0.142119,0.121345,0.0931010,0.123415]
;*** aqua as above and terra original, also binned at 24 km
modpwr_avg = [14.8860,3.15573,0.983821,0.303705,0.182096,0.129001,0.0851479,0.0685612,0.110567,0.0955418,0.0803466,0.0931010,0.0701659]

;*** average of all of aqua spectra binning the frequencies to 24 km
modpwr_A_all = [23.9030,3.03724,1.06171,0.374062,0.187108,0.178776,0.0843073,0.0664581,0.129795,0.101489,0.129237,0.105643,0.0770643]

;*** all of Agua and original Terra binned to 24 km (0.04167)
modpwr_avg_all = [15.7105,3.03724,0.930003,0.280011,0.187108,0.140392,0.0843073,0.0557682,0.129795,0.0752268,0.0842925,0.105643,0.0469905]

;*** leaving Terra original as it is and averaging aqua_minus_1leg to 0.0712286 (14 km):
;modpwr_A = [22.2539,2.16254,0.299393,0.144604,0.0782822,0.132208,0.107586,0.117768]
;modpwr_avg = [14.8860,1.48042,0.242677,0.123306,0.0616802,0.0905860,0.0734668,0.0673423]
;modfreq_A = modfreq

;;; Define solid circles for symbols
;a=findgen(16)*(!pi/8.)
;usersym,cos(a),sin(a),/fill,thick=3
;symsize=1.3

lonmin = -150.00
;lonmax = -149.60
lonmax = -149.50
latmax = 71.96
;latmin = 71.80
latmin = 70.3
;bindis = 1000.
bindis = 10.

radius = 0.005   ;r = 0.5 km
;radius = 0.02   ;r = 2 km
;radius = 0.04   ;r = 4 km
;radius = 0.05   ;r = 5 km
radiust =strcompress(string(radius,format="(f5.3)"),/remove_all)

;open besst data
bes_file = '../BESST2/besstdata_calsc_4bill.txt'  ;considers interpolation periods + what I thinkis bad portions
bl = file_lines(bes_file)
besst_orig = dblarr(5, bl)
openr,unit,bes_file,/get_lun
readf,unit,besst_orig
close,unit
free_lun,unit

;*** Shifting the time series
;shiftval = -7890
shiftval = -7820
;shiftval = -7910
;shiftval = -15680 ;terra
;shiftval = -15600
;shiftval = -15560
besst = besst_orig
besst_size = size(besst_orig)
besst_len = besst_size(2)
besst(0,*) = shift(besst_orig(0,*),shiftval)
besst(3,*) = shift(besst_orig(3,*),shiftval)
besst(4,*) = shift(besst_orig(4,*),shiftval)
;newlen=besst_len+shiftval-1 -23;;rm the last 23 points of the last leg
newlen=besst_len+shiftval-1;;rm the last 23 points of the last leg
besst = besst(*,0:newlen)

okb = where(besst(0,*) ge -1.98,cok)
besst = besst(*,okb)
bes_sst = reform(besst(0,*))
bes_lon = reform(besst(1,*))
bes_lat = reform(besst(2,*))
bes_tme = reform(besst(3,*))
bes_std = besst(4,*)
bes_tme_hr=bes_tme/86400.*24. + bes_lon/15.
tme_delay_hr = (bes_tme(0)-besst_orig(3,0))/86400.*24.
tme_delay_min = (tme_delay_hr-fix(tme_delay_hr))*60.
print,"Tme Delay: ",tme_delay_hr,' ',fix(tme_delay_hr),':',tme_delay_min

tracks = uniq(bes_lon)
unilon = bes_lon(tracks)
ntracks = n_elements(unilon)
ts_sst_all = fltarr(40000)
ts_dist_all = fltarr(40000)
ts_dist_bin = fltarr(40000)
;ts_dist_bin = fltarr(4000)
start = 0
stdis = 0
specarr = replicate(!values.f_nan,2000,7)
specarr_sm = replicate(!values.f_nan,2000,7)
freqarr = replicate(!values.f_nan,2000,7)

;slps = [0.00008,0.0005,0.003,0.003,0.003,0.00045,0.02]
;slps = [0.00008,0.0005,0.00001,0.0008,0.003,0.001,0.02]  ;using a smoothing of 0.0009
slps = [0.00008,0.0005,0.00001,0.0009,0.003,0.001,0.02]  ;using a smoothing of 0.001
slps_sm = [0.000008,0.00005,0.0003,0.003,0.0008,0.000045,0.002]

;OJO! when using cgGrek in legend need to unse /normal!!!!!!!!!!!!
nustr = cgGreek('nu')+'!U-2!N'

;items1 = ['BESST','MODIS','95% Conf']
;items2 = ['BESST','MODIS','!7m!x!U-2!N']
;items2 = ['MODIS','BESST','BESST Smooth','!7m!x!U-2!N']
;items2 = ['MODIS','BESST','BESST Smooth',cgGreek('nu')+'!U-2!N']
items2 = ['MODIS','BESST','BESST Smooth',nustr]
items1 = ['MODIS','BESST','BESST Smooth']
items3 = ['MODIS Terra','MODIS Avg','BESST','BESST Smooth']
;items4 = ['MODIS Terra','MODIS Avg','BESST','BESST Smooth','!7m!x!U-2!N']
items4 = ['MODIS Terra','MODIS Avg','BESST','BESST Smooth',nustr]
items5 = ['MODIS Terra','MODIS Avg','BESST','BESST Smooth','!7m!x!U-2!N','!7m!x!U-3!N']
;items6 = ['MODIS Terra','MODIS Avg','BESST Avg','!7m!x!U-2!N']
;items6 = ['MODIS Terra','MODIS Avg','BESST Avg',nustr]
items6 = ['MODIS Terra','MODIS Avg','BESST Avg','-2 Slope', '-5/3 Slope']
items7 = ['MODIS Terra','MODIS Avg','BESST Avg']
symarr = [-4,88,-3]

for i = 0,ntracks -1 do begin   ;i:track loop

  print,'Doing Track :',unilon(i)
  ;track_id = where(bes_lon eq unilon(i),ctracks)
  track_id = where(bes_lon eq unilon(i) and bes_lat ge 71.6,ctracks)  ;Limit 150 to the domain of the others!!

  match_sst = bes_sst(track_id)
  match_tme = bes_tme(track_id)
  match_lat = bes_lat(track_id)
  match_lon = bes_lon(track_id)
  match_tme_hr=match_tme/86400.*24. + unilon(i)/15.
  match_std = bes_std(track_id)
 
  ;** Need to work with tracks sorted by time
  tmeidx = sort(match_tme)
  latbs = match_lat(tmeidx)
  lonbs = match_lon(tmeidx)
  sstbs = match_sst(tmeidx)
  tmebs_h = match_tme_hr(tmeidx)
  tmebs=match_tme(tmeidx)
  stdbs = match_std(tmeidx)

  ;plot,tmebs,sstbs,psym=3,/ynoz

  ;Where the jumps occur (these are not due to calibration but what where the turns before)
  tmeuid=uniq(tmebs)
  tmeu = tmebs
  ntme = n_elements(tmeu)
  jumps = fltarr(ntme-1)
  for m=0,ntme-2 do jumps(m)=tmeu(m+1)-tmeu(m)  ;m: loop for calibration jumps
  caljump = where(jumps gt 2,njumps)
  cal =[-1,caljump,ntme-1]
  ;plot,jumps
  ;print,njumps

  dt = 0.333
  tmejumps = fix(jumps(caljump))
  intmes = total(fix(jumps(caljump)))/dt
  ngaps = fix(intmes)

  new_ntme = ntme+ngaps-njumps

  new_tmeu = dblarr(new_ntme)
  new_sst = replicate(!values.f_nan,new_ntme)
  new_std = replicate(!values.f_nan,new_ntme)
  new_lat = replicate(!values.f_nan,new_ntme)
  new_lon = replicate(!values.f_nan,new_ntme)

  cnta = 0
  cntj = 0

  for s=0,njumps do begin  

    cntaa = cnta + cal(s+1)-cal(s)
    ;print,cnta,cntaa

    new_tmeu(cnta:cntaa-1) = tmeu(cal(s)+1:cal(s+1))
    new_sst(cnta:cntaa-1) = sstbs(cal(s)+1:cal(s+1))
    new_std(cnta:cntaa-1) = stdbs(cal(s)+1:cal(s+1))
    new_lat(cnta:cntaa-1) = latbs(cal(s)+1:cal(s+1))
    new_lon(cnta:cntaa-1) = lonbs(cal(s)+1:cal(s+1))

    ;* computing time array inside gaps
    if s le njumps-1 then begin
        gap_tme = dblarr(ngaps)
        dum = fix(tmejumps(s)/dt)-1
        test_tme = dblarr(dum)
        cntjj = cntj+dum
        for xx = 0,dum-1 do test_tme(xx)=tmeu(caljump(s))+dt*(xx+1)
        gap_tme(cntj:cntjj-1) = test_tme
        cntj = cntjj
        cnta = cntaa
        cntaa = cnta+dum
        ;print,s,cnta,cntaa-1
        new_tmeu(cnta:cntaa-1) = test_tme
    endif
    cnta = cntaa

  endfor

  can = n_elements(new_sst)
  stp = start+can
  print,'Length sst_interp: ',start,can
 
  lat_interp = new_lat
  lbad = where(finite(lat_interp,/NaN),/L64,cbad)
  lgood = where(finite(lat_interp),/L64,cgood)
  lat_interp(lbad) = interpol(lat_interp(lgood),new_tmeu(lgood),new_tmeu(lbad))

  lon_interp = new_lon
  lbad = where(finite(lon_interp,/NaN),/L64,cbad)
  lgood = where(finite(lon_interp),/L64,cgood)
  lon_interp(lbad) = interpol(lon_interp(lgood),new_tmeu(lgood),new_tmeu(lbad))

  sst_interp = new_sst
  sbad = where(finite(sst_interp,/NaN),/L64,cbad)
  sgood = where(finite(sst_interp),/L64,cgood)
  sst_interp(sbad) = interpol(sst_interp(sgood),new_tmeu(sgood),new_tmeu(sbad))
  ;oplot,new_tmeu,sst_interp,psym=3,color=25
  ;stop

  ;** distance traveled
  ;distance = fltarr(2,can)
  ;for m=0,can-2 do distance(*,m)= map_2points(lon_interp(m+1),lat_interp(m+1),lon_interp(m),lat_interp(m),/meters)
  ;sep = mean(distance(0:can-2))/1000.
  distance = fltarr(can)
  for m=0,can-2 do distance(m)= map_2points(lon_interp(m+1),lat_interp(m+1),lon_interp(m),lat_interp(m),/meters)
  sep = mean(distance(0:can-2))/1000.
  ;print,'distance (km): ',distance(0:can-2)/1000.
  print,'Mean distance (km): ',sep
  print,'length leg (km): ',total(distance(0:can-2))/1000.
  ;window,3
  ;plot,distance(0,*)/1000.,psym=2,/ynoz

  ;binning best to 1 km grids for entire lenght (binned) array
  totdis=total(distance,/cumulative)
  dis_bin=round(totdis/bindis)
  unibin = dis_bin(uniq(dis_bin))
  nu=n_elements(unibin)
  startd = stdis

  ;lat_tst = fltarr(nu)
  ;lon_tst = fltarr(nu)

  for xu=0,nu-1 do begin
    unitst=where(unibin(xu) eq dis_bin,cut)
    meandisbes=mean(sst_interp(unitst))
    ts_dist_bin(stdis)=meandisbes

    ;testing if we have horizontal transects (no. only longitudinal ones)
    ;latbin=mean(lat_interp(unitst))
    ;lonbin=mean(lon_interp(unitst))
    ;lat_tst(xu)=latbin
    ;lon_tst(xu)=lonbin

    stdis=stdis+1  
  endfor
  stopd = stdis-1
  print,'Binned transect: ',startd,stopd
  sst_bin = ts_dist_bin(startd:stopd)
  ;plot,lon_tst,lat_tst,/ynoz,psym=2
  ;stop

  ;** entire length arrays without binning)
  ts_sst_all(start:stp-1) = sst_interp

  ;std_interp = new_std
  ;std_interp(sbad) = interpol(std_interp(sgood),new_tmeu(sgood),new_tmeu(sbad))

  start = stp
  ulonstr = string(unilon(i),format="(f7.2)")
  ulonstr2 = string(abs(unilon(i)),format="(f7.2)")+"W"

  ;** this ones are not in a uniform grid
  ;spec_lin = FFT_PowerSpectrum(sst_interp,sep,FREQ=freq_lin,significance=signif)
  spec_lin = FFT_PowerSpectrum(sst_interp,sep,FREQ=freq_lin,significance=signif,sig_level=0.025)
  ;spec_lin_sm = smooth(spec_lin,3)
  ;spec_lin2 = FFT_PowerSpectrum(sst_interp,sep, FREQ=freq_lin2, significance=signif2,/tukey,width=0.001,sig_level=0.025)
  spec_lin2 = FFT_PowerSpectrum(sst_interp,sep, FREQ=freq_lin2, significance=signif2,/tukey,width=0.0009,sig_level=0.025)

  ;** this ones are in a uniform grid of 10 m
  spec_bin = FFT_PowerSpectrum(sst_bin,0.01,FREQ=freq_bin,significance=signif_bin,sig_level=0.025)
  spec_bin2 = FFT_PowerSpectrum(sst_bin,0.01,FREQ=freq_bin2, significance=signif2_bin,/tukey,width=0.001,sig_level=0.025)
  ns = n_elements(freq_bin)
  specarr(0:ns-1,i)=spec_bin
  specarr_sm(0:ns-1,i)=spec_bin2
  freqarr(0:ns-1,i)=freq_bin

  ;*** What is the other end of the significance level
  N = n_elements(sst_interp)
  siglvl=0.025
  ;signif_orig = float(signif(0))/(CHISQR_CVF(0.975, 2.0)/2.0)
  ;sig_ll = signif_orig*(CHISQR_CVF(siglvl, FLOAT(N*2.0))/FLOAT(N*2.0))^(-1)
  ;sig_ul = signif_orig*(CHISQR_CVF(1.-siglvl, FLOAT(N*2.0))/FLOAT(N*2.0))^(-1)
  ;sig_ll = signif_orig*(CHISQR_CVF(siglvl, 2.0)/2.0)^(-1)
  ;sig_ul = signif_orig*(CHISQR_CVF(1.-siglvl, 2.0)/2.0)^(-1)
  ;sig_ul = alog(2.0)-alog(CHISQR_CVF(1.-siglvl, 2.0))
  ;sig_ll = alog(2.0)-alog(CHISQR_CVF(siglvl, 2.0))
  ;print,'LL Significance: ',sig_ll
  ;print,'UL Significance: ',sig_ul

  ;hwindow = HANNING(can, alpha=0.54, /DOUBLE)
  hwindow = HANNING(can, alpha=0.6, /DOUBLE)
  ;sst_interp_sm=hwindow*sst_interp
  ;spec_lin_sm_han = FFT_PowerSpectrum(sst_interp_sm,sep, FREQ=freq_lin_sm, significance=signif_sm)
  sst_window=hwindow*sst_bin
  spec_lin_sm_han = FFT_PowerSpectrum(sst_window,0.01, FREQ=freq_lin_sm, significance=signif_sm)
  spec_lin_sm_han2 = FFT_PowerSpectrum(sst_window,0.01, FREQ=freq_lin_sm2, /tukey,width=0.001,significance=signif_sm)

  ;window,0
  plotsym,0,1.0,/fill
  device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_log.eps',xsize=8,ysize=6.0,/inches,/color
  plot,freq_lin,spec_lin,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$
    title='Horizontal SST Spectrum Along'+ulonstr2,ytitle='SST Power [(!Uo!NC)!U2!N km]',$
    YTickFormat='exponent'
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  oplot,freq_lin2,spec_lin2,thick=3,color=3
  oplot,modfreq_A,modpwr_A,color=3,psym=-4
  ;;oplot,modfreq_A,modpwr_avg,color=4,psym=-4
  ;;oplot,modfreq_A,modpwr_avg_all,color=4,psym=-4
  ;;oplot,freq_lin,signif,color=4,thick=2
  legend,items1,color=[2,0,3],psym=[-4,88,-3],number=2,/top,/right,/fill,box=1
  device,/close

  ;window,2
  plotsym,0,1.0,/fill
  device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_loglog.eps',xsize=8,ysize=6.0,/inches,/color
  plot,freq_lin,spec_lin,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=8,$
    title='Horizontal SST Spectrum Along'+ulonstr2,ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3,symsize=1.2
  oplot,freq_lin2,spec_lin2,color=3
  ;;oplot,modfreq_A,modpwr_A,color=3,psym=-8
  ;;oplot,modfreq_A,modpwr_avg,color=4,psym=-8
  ;;oplot,modfreq_A,modpwr_avg_all,color=4,psym=-4
  oplot,freq_lin,slps(i)*freq_lin^(-2),linestyle=0,color=4,thick=3
  legend,items2,color=[2,0,3,4],psym=[-4,88,-3,-3],number=2,/top,/right,/fill,box=1
  device,/close

  ;***** binning data to every 10m segments
  ;set_plot,'ps'

  figlabel_log = ['','','','(a)','(c)','(e)','']
  figlabel_loglog = ['','','','(b)','(d)','(f)','']
  thickl =replicate(5,5)

  ;window,0
  plotsym,0,1.0,/fill
  device,/encapsulated,filename='tracks/spectra/fft_10mbin_shift_'+ulonstr+'_log.eps',xsize=8,ysize=6.0,/inches,/color,$
    font_size=15,yoffset=1.5,xoffset=1,set_font='times-Bold',bit=8
  ;plot,freq_bin,spec_bin,/ylog,font=0,XTITLE='!7m!x [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$
  plot,freq_bin,spec_bin,/ylog,font=0,XTITLE=cgGreek('nu')+' [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$
    title='Horizontal, binned SST Spectrum Along'+ulonstr2,ytitle='SST Power [(!Uo!NC)!U2!N km]',$
    YTickFormat='exponent'
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  oplot,freq_bin2,spec_bin2,thick=3,color=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  legend,items1,color=[2,0,3],psym=[-4,88,-3],number=2,/top,/right,/fill,box=1,thick=thickl(0:2),/normal
  if i eq 4 then xyouts,0.15,3.5,'(c)',charsize=1.5 else xyouts,0.15,25,figlabel_log(i),charsize=1.5
  device,/close

  ;window,2
  plotsym,0,1.0,/fill
  device,/encapsulated,filename='tracks/spectra/fft_10mbin_shift_'+ulonstr+'_loglog.eps',xsize=8,ysize=6.0,/inches,/color,$
    font_size=15,yoffset=1.5,xoffset=1,set_font='times-Bold',bit=8;,/helvetica
  ;plot,freq_bin,spec_bin,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,200],xstyle=1,psym=8,$
  plot,freq_bin,spec_bin,/ylog,/xlog,/ynoz,XTITLE=cgGreek('nu')+' [km!U-1!N]',xrange=[0.01,200],xstyle=1,psym=8,$
    title='Horizontal SST Spectrum Along'+ulonstr2,ytitle='SST Power [(!Uo!NC)!U2!N km]',$
    YTickFormat='exponent'
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  oplot,freq_bin2,spec_bin2,thick=3,color=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,freq_bin,slps(i)*freq_bin^(-2),linestyle=0,color=4,thick=3
  legend,items2,color=[2,0,3,4],psym=[-4,88,-3,-3],number=2,/top,/right,/fill,box=1,thick=thickl(0:3),/normal
  ;xyouts,0.013,20,'(b)',charsize=1.5
  xyouts,0.013,20,figlabel_loglog(i),charsize=1.5
  device,/close
  ;set_plot,'x'

  ;*** Working with raw data above instead
  ;window,1
  ;plotsym,0,1.0,/fill
  ;;;device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_log_han.eps',xsize=8,ysize=6.0,/inches,/color
  ;plot,freq_lin_sm,spec_lin_sm_han,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,2],xstyle=1,psym=2,$
  ;  title='Hanning Horizontal SST Spectrum Along '+ulonstr + ' Longitude',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  ;oplot,freq_lin_sm,spec_lin_sm_han
  ;oplot,modfreq,modpwr,color=4,psym=-4
  ;oplot,freq_lin_sm2,spec_lin_sm_han2,thick=3,color=3
  ;;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  ;oplot,modfreq_A,modpwr_A_all,color=3,psym=-4
  ;;oplot,modfreq_A,modpwr_avg,color=4,psym=-8
  ;;oplot,freq_lin_sm,signif_sm,color=4,thick=2
  ;;oplot,freq_lin_sm,spec_lin_sm_han+signif_sm,color=4,thick=1,linestyle=0
  ;;oplot,freq_lin_sm,spec_lin_sm_han-signif_sm,color=3,thick=1,linestyle=0
  ;legend,items1,color=[0,2,4],psym=symarr,number=2,/top,/right,/fill,box=1
  ;stop

  ;window,3
  ;plotsym,0,1.0,/fill
  ;;device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_loglog.eps',xsize=8,ysize=6.0,/inches,/color
  ;plot,freq_lin_sm,spec_lin_sm_han,/ylog,/xlog,/ynoz,$
  ;  XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=2,$
  ;  title='Hanning Horizontal Wavenumber Spectrum Along '+ulonstr+' Longitude',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  ;oplot,modfreq,modpwr,color=4,psym=-4
  ;oplot,freq_lin_sm2,spec_lin_sm_han2,thick=3,color=3
  ;;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  ;oplot,modfreq_A,modpwr_A_all,color=3,psym=-4
  ;;oplot,modfreq_A,modpwr_avg,color=4,psym=-8
  ;oplot,freq_lin,slps_sm(i)*freq_lin^(-2),linestyle=0,color=2,thick=2
  ;legend,items2,color=[0,2,4],psym=symarr2,number=2,/top,/right,/fill,box=1

  ;stop
  
endfor

;set_plot,'x'
;set_plot,'ps'

ts_dist_bin = ts_dist_bin(0:stdis-1)
ts_sst_all = ts_sst_all(0:stp-1)
;ts_sst_all = ts_sst_all(0:stp-1-can)  ;removing the last points corresponding to the last transect
;ts_sst_all = smooth(ts_sst_all(0:stp-1-can),7)

;*** Binning all the besst spectra (like in modis)
;** Bin array
;binsize = 0.025  ;besst
binsize = 0.035
;binsize = 0.03
;binsize = 0.04
;binsize = 0.0555556  ;terra  : 18 km
;binsize = 0.0416667  ;aqua  : 24 km  -- used
;binsize = 0.0714286   ;original terra 14 km (bin aqua and leave terra un binned at the orig res)

;** Remove the row for -149.70
specarr=specarr(*,0:5)

mbinid =  freqarr/binsize
ok = where(finite(mbinid))
mbinid(ok) = round(mbinid(ok))

;*** Average by column
bspecbin = fltarr(1600)
bspecbin2 = fltarr(1600)
frqbin=indgen(1600)*binsize
for i=0,1599 do begin
    idbin = where(mbinid eq i,cid)
    bspecbin(i) = mean(specarr(idbin),/NaN)
    bspecbin2(i) = mean(specarr_sm(idbin),/NaN)
endfor

;******** Paper plot ***********
set_plot,'ps'
;set_plot,'x'

;window,0
plotsym,0,0.8,/fill
;;device,/encapsulated,filename='tracks/spectra/fft_shift_bin_040_log.eps',xsize=8,ysize=6.0,/inches,/color
;;device,/encapsulated,filename='tracks/spectra/fft_shift_bin_035_log.eps',xsize=8,ysize=6.0,/inches,/color
;device,/encapsulated,filename='plots/paper/castro_fig9a_25m_bottomlegend.eps',xsize=8,ysize=6.0,/inches,/color,$
device,/encapsulated,filename='plots/paper/castro_fig9a_5m_nolegend.eps',xsize=8,ysize=6.0,/inches,/color,$
    font_size=15,yoffset=1.5,xoffset=1,bit=8,set_font='times-Bold'
;plot,frqbin,bspecbin,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$
;plot,frqbin,bspecbin,/ylog,XTITLE= cgGreek('nu')+' [km!U-1!N]',xrange=[0,25],xstyle=1,psym=8,$
plot,frqbin,bspecbin,/ylog,XTITLE= cgGreek('nu')+' [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$  ;zoom in
  title='BESST SST Blended Spectrum',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  ;oplot,frqbin,bspecbin2,color=3
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  ;legend,items1,color=[2,0,3],psym=[-4,88,-3],number=2,/top,/right,/fill,box=1
  ;legend,items3,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1,thick=thickl(0:3)
  ;legend,items7,color=[2,5,0],psym=[-4,-4,88],number=2,/top,/right,/fill,box=1,thick=thickl(0:2)
  ;legend,items7,color=[2,5,0],psym=[-4,-4,88],number=2,/bottom,/left,/fill,box=1,thick=thickl(0:2)
  xyouts,0.15,35,'(a)',charsize=1.5
device,/close

t1= where(10^alog10(frqbin) le 13,ct1)
t2= where(10^alog10(frqbin) gt 15 and 10^alog10(frqbin) le 40,ct2)

;window,2
plotsym,0,0.8,/fill
;;device,/encapsulated,filename='tracks/spectra/fft_shift_bin_040_loglog.eps',xsize=8,ysize=6.0,/inches,/color
device,/encapsulated,filename='plots/paper/castro_fig9b_m.eps',xsize=8,ysize=6.0,/inches,/color,$
    font_size=15,yoffset=1.5,xoffset=1,set_font='times-Bold',bit=8;,/helvetica
;;plot,frqbin,bspecbin,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=8,$
plot,frqbin,bspecbin,/ylog,/xlog,/ynoz,XTITLE= cgGreek('nu')+' [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=8,$
  title='BESST SST Blended Spectrum',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  ;oplot,frqbin,bspecbin2,color=3
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  ;oplot,frqbin,0.0009*frqbin^(-2),linestyle=0,color=6,thick=3
  oplot,frqbin(t1),0.00095*frqbin(t1)^(-2),linestyle=0,color=6,thick=3
  ;oplot,frqbin(t2),0.0003*frqbin(t2)^(-5./3.),linestyle=0,color=5,thick=3
  oplot,frqbin(t2),0.00065*frqbin(t2)^(-5./3.),linestyle=0,color=4,thick=5
  oplot,frqbin(t2),0.000085*frqbin(t2)^(-5./3.),linestyle=0,color=4,thick=5
  legend,items6,color=[2,5,0,6,4],psym=[-4,-4,88,-3,-3],number=2,/top,/right,/fill,box=1,spacing=1.5,thick=thickl(0:4),/normal
  xyouts,0.013,20,'(b)',charsize=1.5
device,/close
set_plot,'x'
print,'stop here'
stop



;sep4 = 0.01   ;orig nothing
sep4 = 0.001  ;orig hanning
;sep4 = 0.005  ;orig hanning
;sep4 = 0.007  ;mean of all the distances screws up modis with hanning
sep5 = 0.01

;** originally had a distance of 0.01, but in the hanning we used 0.001.  Which is right?
spec_lin_all = FFT_PowerSpectrum(ts_sst_all,sep4, FREQ=freq_lin_all, significance=signif_all,sig_level= 0.025)
spec_lin2_all = FFT_PowerSpectrum(ts_sst_all,sep4, FREQ=freq_lin2_all, significance=signif2_all,/tukey,width=0.0001,sig_level=0.025)

spec_bin_all = FFT_PowerSpectrum(ts_dist_bin,sep4, FREQ=freq_dis_bin, significance=signif_bin,sig_level= 0.025)
spec_bin2_all = FFT_PowerSpectrum(ts_dist_bin,sep4, FREQ=freq_dis2_bin, significance=signif2_bin,/tukey,width=0.005,sig_level= 0.0025)
;spec_bin_all = FFT_PowerSpectrum(ts_dist_bin,1, FREQ=freq_dis_bin, significance=signif_bin,sig_level= 0.025)
;spec_bin2_all = FFT_PowerSpectrum(ts_dist_bin,1, FREQ=freq_dis2_bin, significance=signif2_bin,/tukey,width=0.06,sig_level= 0.025)

window,0
plotsym,0,1.0,/fill
;device,/encapsulated,filename='tracks/spectra/fft_shift_region_log.eps',xsize=8,ysize=6.0,/inches,/color
plot,freq_dis_bin,spec_bin_all,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$
  title='Horizontal SST Spectrum, binned 1km',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,freq_dis2_bin,spec_bin2_all,color=3
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  ;legend,items1,color=[2,0,3],psym=[-4,88,-3],number=2,/top,/right,/fill,box=1
  ;legend,items1,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1
  legend,items3,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1
;device,/close

window,2
plotsym,0,1.0,/fill
;device,/encapsulated,filename='tracks/spectra/fft_shift_region_log.eps',xsize=8,ysize=6.0,/inches,/color
plot,freq_dis_bin,spec_bin_all,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=8,$
  title='Horizontal SST Spectrum, binned 1km',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,freq_dis2_bin,spec_bin2_all,color=3
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  oplot,freq_dis_bin,0.005*freq_dis_bin^(-2),linestyle=0,color=4,thick=3
  oplot,freq_dis_bin,0.01*freq_dis_bin^(-3),linestyle=0,color=6,thick=2
  ;oplot,freq_dis_bin,spec_bin_all_sm,color=4
  ;oplot,freq_dis_bin,signif_all,color=4,thick=2
  ;legend,items1,color=[2,0,3],psym=[-4,88,-3],number=2,/top,/right,/fill,box=1
  ;legend,items1,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1
  legend,items3,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1
;device,/close
stop

window,0
plotsym,0,1.0,/fill
;device,/encapsulated,filename='tracks/spectra/fft_shift_region_log.eps',xsize=8,ysize=6.0,/inches,/color
plot,freq_lin_all,spec_lin_all,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,5],xstyle=1,psym=8,$
;plot,freq_lin_all,spec_lin_all,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,5],xstyle=1,psym=-8,$
  title='Horizontal SST Spectrum ',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,freq_lin2_all,spec_lin2_all,color=3
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  ;oplot,freq_lin_all,spec_lin_all_sm,color=4
  ;oplot,freq_lin_all,signif_all,color=4,thick=2
  ;legend,items1,color=[2,0,3],psym=[-4,88,-3],number=2,/top,/right,/fill,box=1
  ;legend,items1,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1
  legend,items3,color=[2,5,0,3],psym=[-4,-4,88,-3],number=2,/top,/right,/fill,box=1
;device,/close

;window,2
;set_plot,'ps'
plotsym,0,1.0,/fill
;device,/encapsulated,filename='tracks/spectra/fft_shift_region_loglog.eps',xsize=8,ysize=6.0,/inches,/color
;device,/encapsulated,filename='tracks/spectra/fft_shift_region_loglog_v2.eps',xsize=8,ysize=6.0,/inches,/color
plot,freq_lin_all,spec_lin_all,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,1000],xstyle=1,psym=8,$
;plot,freq_lin_all,spec_lin_all,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=8,$
;plot,freq_lin_all,spec_lin_all,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,11],xstyle=1,psym=8,$
    title='Horizontal Wavenumber Spectrum',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,modfreq,modpwr,color=2,psym=-4,thick=3
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  oplot,freq_lin2_all,spec_lin2_all,color=3
  ;oplot,freq_lin_all,0.0005*freq_lin_all^(-2),linestyle=0,color=4,thick=2
  oplot,freq_lin_all,0.003*freq_lin_all^(-2),linestyle=0,color=4,thick=3
  ;oplot,freq_lin_all,0.003*freq_lin_all^(-3),linestyle=0,color=6,thick=2
  oplot,freq_lin_all,0.004*freq_lin_all^(-3),linestyle=0,color=6,thick=2
  ;oplot,freq_lin_all,0.0003*freq_lin_all^(-3),linestyle=0,color=6,thick=2
  ;legend,items2,color=[0,2,4],psym=symarr2,number=2,/top,/right,/fill,box=1
  ;legend,items4,color=[2,5,0,3,4],psym=[-4,-4,88,-3,-3],number=2,/top,/right,/fill,box=1
  legend,items5,color=[2,5,0,3,4,6],psym=[-4,-4,88,-3,-3,-3],number=2,/top,/right,/fill,box=1,spacing=1.5
;device,/close
;set_plot,'x'
stop

;set_plot,'x'

;ts_all_sm = smooth(ts_sst_all,7)
;spec_lin_all_sm = FFT_PowerSpectrum(ts_all_sm,sep4, FREQ=freq_lin_all_sm, significance=signif_all_sm,sig_level= 0.025)
;spec_lin2_all_sm = FFT_PowerSpectrum(ts_all_sm,sep4, FREQ=freq_lin2_all_sm, significance=signif2_all_sm,/tukey,width=0.0001,sig_level=0.025)
;
;plotsym,0,1.0,/fill
;window,1
;;device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_log.eps',xsize=8,ysize=6.0,/inches,/color

;plot,freq_lin_all_sm,spec_lin_all_sm,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,10],xstyle=1,psym=8,$
;  title='Horizontal SST Spectrum ',ytitle='SST Power [(!Uo!NC)!U2!N km]'
;  oplot,modfreq,modpwr,color=2,psym=-4
;  oplot,modfreq_A,modpwr_A,color=3,psym=-4
;  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
;  oplot,freq_lin2_all_sm,spec_lin2_all_sm,color=3
;  ;oplot,freq_lin_all,signif_all,color=4,thick=2
;  legend,items1,color=[0,2,4],psym=symarr,number=2,/top,/right,/fill,box=1

;window,3
;;device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_loglog.eps',xsize=8,ysize=6.0,/inches,/color
;plot,freq_lin_all_sm,spec_lin_all_sm,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,100],xstyle=1,psym=8,$
;    title='Horizontal Wavenumber Spectrum',ytitle='SST Power [(!Uo!NC)!U2!N km]'
;  oplot,modfreq,modpwr,color=2,psym=-8
;  oplot,freq_lin2_all_sm,spec_lin2_all_sm,color=3
;  oplot,freq_lin_all_sm,0.0005*freq_lin_all_sm^(-2),linestyle=0,color=4,thick=2
;  ;oplot,freq_lin_all_sm,0.0007*freq_lin_all_sm^(-3),linestyle=0,color=5,thick=2
;  ;legend,items2,color=[0,2,4],psym=symarr2,number=2,/top,/right,/fill,box=1
;  legend,items2,color=[2,0,3,4],psym=[-4,88,-3,-3],number=2,/top,/right,/fill,box=1

;stop

;*** Is there some scaling going on here???

;spec_lin_all = FFT_PowerSpectrum(ts_sst_all,0.001, FREQ=freq_lin_all, significance=signif_all)
;spec_lin_all_sm = smooth(spec_lin_all,5)
;ts_sst_all_sm = smooth(ts_sst_all,5)

nt=n_elements(ts_sst_all)
hwindow = HANNING(nt, /DOUBLE)
ts_sst_all_han=hwindow*ts_sst_all
spec_lin_all_han = FFT_PowerSpectrum(ts_sst_all_han,sep4, FREQ=freq_lin_all_han, significance=signif_all_han,sig_level= 0.025)
spec_lin2_all_han = FFT_PowerSpectrum(ts_sst_all_han,sep4, FREQ=freq_lin2_all_han, significance=signif2_all_han,sig_level= 0.025,/tukey,width=0.0001)

window,1
plotsym,0,1.0,/fill
;device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_log.eps',xsize=8,ysize=6.0,/inches,/color

;plot,freq_lin_all,spec_lin_all,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,10],xstyle=1,psym=-8,$
plot,freq_lin_all_han,spec_lin_all_han,/ylog,XTITLE='!7m!x [km!U-1!N]',xrange=[0,10],xstyle=1,psym=8,$
  title='Horizontal SST Spectrum ',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,modfreq,modpwr,color=2,psym=-4
  ;oplot,modfreq_A,modpwr_A,color=3,psym=-4
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  oplot,freq_lin2_all_han,spec_lin2_all_han,color=3
  ;oplot,freq_lin_all,signif_all,color=4,thick=2
  legend,items1,color=[0,2,4],psym=symarr,number=2,/top,/right,/fill,box=1

window,3
;device,/encapsulated,filename='tracks/spectra/fft_shift_'+ulonstr+'_loglog.eps',xsize=8,ysize=6.0,/inches,/color
;plot,freq_lin_all,spec_lin_all,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,1000],xstyle=1,psym=8,$
plot,freq_lin_all_han,spec_lin_all_han,/ylog,/xlog,/ynoz,XTITLE='!7m!x [km!U-1!N]',xrange=[0.01,1000],xstyle=1,psym=8,$
    title='Horizontal Wavenumber Spectrum',ytitle='SST Power [(!Uo!NC)!U2!N km]'
  oplot,modfreq,modpwr,color=2,psym=-8
  oplot,modfreq_A,modpwr_avg,color=5,psym=-4
  oplot,freq_lin2_all_han,spec_lin2_all_han,color=3
  oplot,freq_lin_all_han,0.005*freq_lin_all_han^(-2),linestyle=0,color=4,thick=2
  oplot,freq_lin_all_han,0.0003*freq_lin_all_han^(-3),linestyle=2,color=4,thick=2
  ;legend,items2,color=[0,2,4],psym=symarr2,number=2,/top,/right,/fill,box=1
  legend,items2,color=[2,0,3,4],psym=[-4,88,-3,-3],number=2,/top,/right,/fill,box=1

stop
set_plot,'x'
print,'Done'
stop

end
