;  Code to expand a 2-D array by a fixed multiple of grid points, interpolating
;  values from the original grid

pro rebin_nan_2d, in, out, mult

;  in:  input array of size (l,m,n)
;  out: interpolated output array of size (l*mult,m*mult,n*mult)
;  mult:  multiplier for size of output array

;  Determine size of input array

in_size = size(in)

if (in_size[0] ne 2) then begin
   print, 'ERROR:  rebin_nan_2d only operates on a 2-D array'
   return
endif

l = in_size[1]
m = in_size[2]

ll = l*mult
mm = m*mult

;  Initialize output array to NaN values

out = fltarr(ll,mm)
out[*,*] = !values.f_nan


;  Begin loop through all the points in the output array.  Done by looping
;  first through original points and then the multiplier for each point

for i = 1, l do begin
for j = 1, m do begin

   for ii = 1, mult do begin
   for jj = 1, mult do begin

      ;  Specify actual index in output array

      index_ii = (i-1)*mult + (ii-1)
      index_jj = (j-1)*mult + (jj-1)

      ;  Determine fractional grid position for current point

      frac_i = ((ii*2.)-1.)/(mult*2.) + (i - 0.5)
      frac_j = ((jj*2.)-1.)/(mult*2.) + (j - 0.5)

      ;  Determine the original points and weights to be used in the interpolation

      if (frac_i ge 1 and frac_i le l) then begin
         u1 = floor(frac_i)
         u2 = ceil(frac_i)
	 weight_u1 = 1.0 - abs(frac_i - u1)
	 weight_u2 = 1.0 - abs(frac_i - u2)
      endif else begin
         if (frac_i lt 1) then begin
	    u1 = 1
	    u2 = 2
	    weight_u1 = 1.0
	    weight_u2 = 0.0
	 endif else begin
	    u1 = l-1
	    u2 = l
	    weight_u1 = 0.0
	    weight_u2 = 1.0
	 endelse
      endelse

      if (frac_j ge 1 and frac_j le m) then begin
         v1 = floor(frac_j)
         v2 = ceil(frac_j)
	 weight_v1 = 1.0 - abs(frac_j - v1)
	 weight_v2 = 1.0 - abs(frac_j - v2)
      endif else begin
         if (frac_j lt 1) then begin
	    v1 = 1
	    v2 = 2
	    weight_v1 = 1.0
	    weight_v2 = 0.0
	 endif else begin
	    v1 = m-1
	    v2 = m
	    weight_v1 = 0.0
	    weight_v2 = 1.0
	 endelse
      endelse

      ;  Initialize sum and count for interpolating between defined values

      val_sum = 0.0
      wgt_sum = 0.0

      val = in[u1-1,v1-1]
      if (finite(val)) then begin
	 weight = weight_u1*weight_v1
         val_sum = val_sum + val*weight
	 wgt_sum = wgt_sum + weight
      endif

      val = in[u1-1,v2-1]
      if (finite(val)) then begin
	 weight = weight_u1*weight_v2
         val_sum = val_sum + val*weight
	 wgt_sum = wgt_sum + weight
      endif

      val = in[u2-1,v1-1]
      if (finite(val)) then begin
	 weight = weight_u2*weight_v1
         val_sum = val_sum + val*weight
	 wgt_sum = wgt_sum + weight
      endif

      val = in[u2-1,v2-1]
      if (finite(val)) then begin
	 weight = weight_u2*weight_v2
         val_sum = val_sum + val*weight
	 wgt_sum = wgt_sum + weight
      endif

      if (wgt_sum gt 0) then begin
         out[index_ii,index_jj] = val_sum/wgt_sum
      endif

      ;print,index_ii,index_jj,frac_i,frac_j,u1,u2,v1,v2,weight_u1,weight_u2,weight_v1,weight_v2

   end
   end     ; end loop through multiplier over original points

end
end     ; End loop through original points

end
