;
;  RGRID computes 2D regular grid from irregular gridded data by binning.
;
;  INPUT
;
;        x...... vector of x location (longitude)
;        y...... vector of y location (latitude)
;        d...... vector of data values to be binned
;        xgrid..... vector of x grid locations
;        ygrid..... vector of y grid locations
;
;  INPUT KEYWORDS:
;                  endpoints  ->  Bins values outside of x,y vectors at the endpoint
;
;  OUTPUT
;        dgrid - 3D array of gridded values (nx,ny,3).  
;
;        Index 1 are a sum of all data points for the grid box
;        Index 2 are a sum of the square of each data point
;        Index 3 gives the population for each grid box.
;
;  NOTES
;        If xgrid or ygrid have cyclic boundary condition, you need
;        to be careful if edge of grid box crosses zero.  Example: if
;        xgrid are longitude values and you want grid box centered on
;        zero (GM), you will need to define grid box for 360 and sum
;        the 0 and 360 grid box.
;
;        Grid boxes with no data are assigned -999 for the sum and squared
;        fields.  The population data are assigned 0.
;
;        Use TRIANGULATE and TRIGRID to interpolate irregularly spaced
;        data to grid
;
;        Data points outside of the boundaries of xgrid and ygrid are not
;        skipped.
;

FUNCTION rgrid, x, y, data, xgrid, ygrid, endpoints=endpoints

result=size(x)
nx=result(1)
result2=size(y)
result3=size(data)
if (nx ne result2(1) or nx ne result3(1) ) then begin
   print, 'regrid:  x, y and data not same length'
   print, 'x length = ',nx
   print, 'y length = ',result2(1)
   print, 'data length = ',result3(1)
   stop
endif
;
; Determine size of 2D grid and define dgrid array
;
result=size(xgrid)
nxg=result(1)
result=size(ygrid)
nyg=result(1)
dgrid=fltarr(nxg,nyg,3)
;
; Bin the irregular spaced data onto grid
;
i=0L
for i = 0L, nx-1L do begin
   IF(KEYWORD_SET(endpoints)) THEN BEGIN
     ix=nearest_index(xgrid,x(i),/endpoints)
     iy=nearest_index(ygrid,y(i),/endpoints)
   ENDIF ELSE BEGIN
     ix=nearest_index(xgrid,x(i),/nomessage)
     iy=nearest_index(ygrid,y(i),/nomessage)
   ENDELSE
   if(ix ne -1 and iy ne -1) then begin
     dgrid(ix,iy,0)=dgrid(ix,iy,0)+data(i)
     dgrid(ix,iy,1)=dgrid(ix,iy,1)+data(i)^2.
     dgrid(ix,iy,2)=dgrid(ix,iy,2)+1.
   endif
endfor

RETURN, dgrid
end
